;;; -*- clips -*-
;;;======================================================
;;;   disposeRules.clp  Ver1.01
;;;
;;;======================================================
;;;

(set-node-index-hash 3)		;for tuningidefault:101j
(set-multithreaded-io TRUE)	;for tuning

;; nakagawaǉ
(defglobal ?*plan-counter* = 1) 
(defglobal ?*component-counter* = 1) 
(defglobal ?*maxDustNum* = 1) 
(defglobal ?*maxBattery* = 100) 

               
;;;*************
;;;* TEMPLATES FOR ROBOT*
;;;*************

(deftemplate robot 
   (slot location (default 0-0))
;   (slot x (default 0))
;   (slot y (default 0))
   (slot battery 
      (default high))
;      (default 100))
;   (slot holding 
;      (default nothing))
   (slot dust-num 	
      (default 0))	
   (slot mode 
      (default dispose-dust)))

(deftemplate object 
   (slot name)
   (slot type 
      (default dust)) 
   (slot location)
;   (slot x (default 0))
;   (slot y (default 0))
   (slot appearance 
      (default nil)) 
       (slot weight 
      (default light)))
                    
(deftemplate goal-is-to 
   (slot action)
   (slot argument-1)
   (slot argument-2))

   
(deftemplate plan 
   (slot number)
   (slot action)
   (slot argument-1)
   (slot argument-2))
   
(deftemplate assignment 
   (slot number)
   (slot component)
   (slot service))

(deftemplate event
    (slot type)		;event type (e.g., cleaning-failed)
    (slot action)	;action causing event (e.g., sweep-dust)
    (slot object)	;target object (e.g., dust)
    (slot location)	;place where event occured
   )               

; e.g. (component (name DiscoverObjectByCamera) (service find) (priority high))
(deftemplate component
    (slot name)		;component name (e.g., DiscoverObjectByCamera)
    (slot service)	;providing service (e.g., find) ;same name as action in plan
    (slot priority (default low))	;priority among same service providing components
   )

(deftemplate cleaning-method 
    (slot method)
    (slot appearance))


;;;*************************
;;;* UTILITIES *
;;;*************************

(deffunction add-plan (?action ?arg-1 ?arg-2) "add plan as a fact and increase counter"
    (assert (plan (number ?*plan-counter*) (action ?action) (argument-1 ?arg-1) (argument-2 ?arg-2)))
    (bind ?*plan-counter* (+ ?*plan-counter* 1))
    )

(deffunction assign-component (?comp ?service) "assign component as a fact and increase component counter"
    (assert (assignment (number ?*component-counter*) (component ?comp) (service ?service)))
    (bind ?*component-counter* (+ ?*component-counter* 1))
    )
                

;;;********************
;;;* FIND OBJECT RULES * 
;;;********************

(defrule find-nearest-object ""
	?find-goal <- (goal-is-to (action find) (argument-1 ?type)(argument-2 nil))
	?move-goal <- (goal-is-to (action move-to) (argument-1 ?type))
    (object (type ?type) (location ?place))
	?obj <- (object (type ?type))
;	(not (goal-is-to (action find) (argument-1 ?place)))
	=>
;	(assert (goal-is-to (action find) (argument-1 ?place))))
  	(printout t "Robot finds " ?type " at " ?place "." crlf)
	(add-plan find ?type nil)
	(modify ?move-goal (argument-1 nil) (argument-2 ?place))
;   	(assert (goal-is-to (action move-to-the-point) (argument-1 ?place)))
	(retract ?find-goal))


;;;***********************
;;;* MOVE                *
;;;***********************

(defrule move-to-target-to-satisfy-task ""
	?goal <- (goal-is-to (action move-to) (argument-1 ?type)(argument-2 nil))
	(not (goal-is-to (action find) (argument-1 ?type)))
	=>
    (assert (goal-is-to (action find) (argument-1 ?type))))



(defrule move-to-target ""
  ?goal <- (goal-is-to (action move-to) (argument-1 nil) (argument-2 ?place))
  ?robot <- (robot (location ~?place))
  =>
  (printout t "Robot moves to " ?place "." crlf)
  (add-plan move-to ?place nil)
  (modify ?robot (location ?place))
  (retract ?goal))


;;;********************
;;;* DISPOSE DUST RULES * 
;;;********************

(defrule move-to-dust-to-dispose ""
  (goal-is-to (action dispose) (argument-1 ?type))
  (robot (location ?place) (mode dispose-dust))
  (not (object (type ?type) (location ?place))) 
;  (not (goal-is-to (action find)))
  (not (goal-is-to (action move-to)))
           =>
  (assert (goal-is-to (action move-to) (argument-1 dust))))
;  (assert (goal-is-to (action move-to) (argument-1 ?place))))



(defrule dispose-dust ""
  ?goal <- (goal-is-to (action dispose) (argument-1 ?type)) 
  ?robot <- (robot (location ?place) (mode dispose-dust) (dust-num ?dust-num)) 
  ?obj <- (object (type ?type) (location ?place))
  =>
  (printout t "Robot disposes dust at " ?place "." crlf)
  (add-plan dispose-dust-at ?place nil)
  (bind ?dust-num (+ ?dust-num 1))
  (modify ?robot (dust-num ?dust-num)) 
    (retract ?obj)
  (retract ?goal))



;;;********************
;;;* DETERMINE CLEANING METHOD RULES * 
;;;********************

(defrule decide-cleaning-method ""
    (declare (salience 15))
	?robot <- (robot)
    ?dust <- (object (type dust) (appearance ?appearance))
;    (event (type cleaning-failed) (action ?failed-action)) 
;    (cleaning-method (method ?new-action&:(<> ?new-action ?failed-action)))
    ?method <- (cleaning-method (method ?action) (appearance ?appearance))    
    ?dispose-goal <- (goal-is-to (action dispose) (argument-1 dust))
	(not (goal-is-to (action charge) (argument-1 battery)))
	=>
	(printout t "Robot decides " ?action " as a cleaning method." crlf)
	(add-plan ?action nil nil)
;    (add-plan dispose-dust-by-using ?action nil)
;	(store CLEANING-METHOD ?action)
	(assert (goal-is-to (action set-CLEANING-METHOD)))
    (modify ?robot (mode cleaning))
    ;  (modify ?robot (battery full) (mode dispose-dust)) 
    (retract ?dispose-goal))

(defrule decide-cleaning-method-in-unexpected-case ""
    (declare (salience 15))
	?robot <- (robot)
    (event (type cleaning-failed) (action ?failed-action)) 
    (cleaning-method (method ?new-action&:(<> ?new-action ?failed-action)))
;    (cleaning-method ?new-action)
;    (neq ?failed-action ?new-action)
    ?dispose-goal <- (goal-is-to (action dispose) (argument-1 dust))
	(not (goal-is-to (action charge) (argument-1 battery)))
	=>
	(printout t "Robot changes cleaning method to " ?new-action "." crlf)
	(add-plan ?new-action nil nil)
;	(add-plan dispose-dust-by-using ?new-action nil)
;	(store CLEANING-METHOD ?new-action)
 	(assert (goal-is-to (action set-CLEANING-METHOD)))
    (modify ?robot (mode change-cleaning-method))
    ;  (modify ?robot (battery full) (mode dispose-dust)) 
	(retract ?dispose-goal))

;;;********************
;;;* ASSIGN COMPONENTS RULES * 
;;;********************

(defrule decide-to-assign-components ""
    (declare (salience 5)) 
    ?plan <- (plan (action ?service))
;    ?component <- (component (service not ?service))
 	(not (assignment (service ?service)))
	(not (goal-is-to (action assign-component) (argument-1 ?service)))
    =>
 	(assert (goal-is-to (action assign-component) (argument-1 ?service))))



(defrule assign-high-priority-component ""
    ?goal <- (goal-is-to (action assign-component) (argument-1 ?service))
    ?component <- (component (name ?comp-name)(service ?service) (priority high))
	(not (event (type component-fault) (object ?comp-name))) 
	=>
	(printout t "Assign component " ?comp-name "for realize " ?service " service." crlf)
	(assign-component ?comp-name ?service)
	(retract ?goal))

(defrule assign-low-priority-component ""
    (declare (salience 5)) 
    ?goal <- (goal-is-to (action assign-component) (argument-1 ?service))
    ?component <- (component (name ?comp-name)(service ?service))
	(not (assignment (service ?service)))
   	(not (event (type component-fault) (object ?comp-name))) 
	=>
	(printout t "Assign component " ?comp-name "for realize " ?service " service." crlf)
	(assign-component ?comp-name ?service)
	(retract ?goal))

(defrule set-cleaning-component ""
    ?goal <- (goal-is-to (action set-CLEANING-METHOD))
    (assignment (component ?comp-name)(service ?service))
    (cleaning-method (method ?service))
	=>
    (printout t "set " ?comp-name " as CLEANING-METHOD." crlf)
    (store CLEANING-METHOD ?comp-name)
;	(assign-component ?comp-name ?service)
	(retract ?goal))

;;;**********************
;;;* INITIAL STATE RULE * 
;;;**********************

(defrule startup ""
  =>
;  (assert (robot (location 0-0) (battery full)))
 ; (assert (goal-is-to (action charge) (argument-1 battery)))
    ;Facts for cleaning methods
	(assert (cleaning-method (method sweep) (appearance LITTER)))
	(assert (cleaning-method (method sweep) (appearance PILE)))
	(assert (cleaning-method (method wipe) (appearance PUDDLE)))
    (assert (cleaning-method (method pick-up) (appearance CAN)))
    (assert (goal-is-to (action dispose) (argument-1 dust)))
;	(assert (object (name leaf) (location X1-Y2) (weight light)))
;   (assert (object (name station) (type station) (location X2-Y2) (weight heavy)))
;	(assert (object (name dustBin) (type dustBin) (location X3-Y3) (weight heavy)))
    ;Facts for components
	(assert (component (name DiscoverObjectByCamera) (service find) (priority high)))
	(assert (component (name DiscoverObjectByRandomWalk) (service find)))
	(assert (component (name MoveTo) (service move-to)))
	(assert (component (name SweepDust) (service sweep)))
	(assert (component (name WipeDust) (service wipe)))
    (assert (component (name PickUpDust) (service pick-up)))
    )


(defglobal ?*time* = (time))
(set-reset-globals FALSE)


(deffunction run-n-times (?n)
  (while (> ?n 0) do
         (reset)
;         (watch activations) 
         (watch rules)
         (run)
         (facts)
         (bind ?n (- ?n 1))))



(defrule setMode
    (robot (mode ?mode))
    =>
	(store MODE ?mode)) 
 

(deffunction prepare ()
         (reset)
;         (watch activations)
         (watch rules))

(deffunction run-once ()
;    (run)
    (store FIRED-RULES (run))
    (facts)
;    (bind ?elapsed-time (- (time) ?*time*)) 
;;	(printout t "Elapsed time: " ?elapsed-time crlf)
;    (store TIME ?elapsed-time)
;	(store RULES (bind ?plans (matches (plan))
    )

(prepare)
;(printout t "Elapsed time: " (integer (- (time) ?*time*)) crlf)



