package org.dyncle;

import java.util.ArrayList;
import java.util.List;



/**
 * Class for storing the information of individual control loops.
 * @author h-nakagawa
 *
 */
public class CLInfo{
	
	private String clName; //name of the control loop (=CL)
	private ComponentBehaviour rootComp; //AD type component of the CL
	private String fqn; //FQN (fully qualified name) of AD type component
	private int priority; //lower is stronger

	/** List of conflict resources (The resources are represented by String) */
	private List<String> confResources; 
	

	public CLInfo(){
		super();
		priority = Integer.MAX_VALUE; //Set weakest priority if not specified.
		confResources = new ArrayList<String>();
	}


	public CLInfo(String name, ComponentBehaviour cb){
		this();
		clName = name;
		rootComp = cb;
	}

	public CLInfo(String name, String afqn, ComponentBehaviour cb, int pri, List<String> confList){
		this(name, cb);
		priority = pri;
		fqn = afqn;
		if(confList!=null){
			for(String st: confList){
				confResources.add(st);
			}
		}
	}

	/**
	 * Resets the member variable's reference.
	 * This method doesn't need to use explicitly for GC.
	 */
	public void remove(){
		rootComp = null;
		confResources = null;
	}

	public String getClName() {
		return clName;
	}


	public void setClName(String clName) {
		this.clName = clName;
	}


	public ComponentBehaviour getRootComp() {
		return rootComp;
	}


	public void setRootComp(ComponentBehaviour rootComp) {
		this.rootComp = rootComp;
	}


	public int getPriority() {
		return priority;
	}


	public void setPriority(int priority) {
		this.priority = priority;
	}


	public List<String> getConfResources() {
		return confResources;
	}


	public void setConfResources(List<String> confResources) {
		this.confResources = confResources;
	}


	public void printValues(){
		System.out.println("\tName: "+clName);
		System.out.println("\tFQN: "+fqn);
		System.out.println("\tpriority: "+priority);

		System.out.print("\tConflict Resources: ");
		for(String str: confResources){
			System.out.print(str+"\t");
		}
		System.out.println("\n");
	}
}
