package org.dyncle;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.List;


/**
 * Control loop management table
 * @author h-nakagawa
 *
 */
public class CLTable{
//	private static CLTable instance = new CLTable();
	
	/**
	 * 	String: control loop's name
	 * 	CLInfo: individual control loop's data
	 */
	public Map<String, CLInfo> clTable;
	

	public CLTable(){
		super();
		clTable = new HashMap<String, CLInfo>();
	}

//	public static CLTable getInstance(){
//		return instance;
//	}
	
//	public void initialize(){
//		clList = new ArrayList<CLInfo>();
//	}
	
	public void addCLInfo(String name, String fqn, ComponentBehaviour cb, 
			int pri, List<String> confList){
		clTable.put(name, new CLInfo(name, fqn, cb, pri, confList));
	}

	public void removeCLInfo(String name){
//		CLInfo clInfo = clTable.get(name);
//		clInfo.remove();
		clTable.remove(name);
	}


	/**
	 * Finds and returns the information of control loop whose name is "name".
	 * @param name name of the control loop
	 * @return corresponding control loop's information (CLInfo object)
	 */
	public CLInfo getCLInfo(String name){
		return clTable.get(name);
	}
	
	/**
	 * Returns AD type components whose control loop's priority is lower than the specified one.
	 * @param name name of the control loop that should be compared with
	 * @return@List of components whose control loop's priority is lower than the specified one.
	 */
	public List<ComponentBehaviour> getLowerPriorityCLs(String name){
		CLInfo baseCL = getCLInfo(name);
		int basePriority = baseCL.getPriority();
		List<String> baseConfResources = baseCL.getConfResources();
		System.out.println("Base priority: "+basePriority);

		List<ComponentBehaviour> resultList = new ArrayList<ComponentBehaviour>();
		for(CLInfo cl: clTable.values()){
//		for(CLInfo cl: clList){
			//compare priority first
			if(basePriority < cl.getPriority()){ //target CL's priority is lower

				//Check whether the control loop has conflict resources
				List<String> comparedConfResources = cl.getConfResources();
				boolean isContained = false;
				for(String baseResource: baseConfResources){
					if(comparedConfResources.contains(baseResource)){
						isContained = true;
					}
				}
				if(isContained) resultList.add(cl.getRootComp());
			}
		}
		return resultList;
	}
	
	/**
    * Checks whether the specified CL can transit to the activate state or not.
    * If the other control loop with higher priority is running, 
    * the control loop can't transit to the activate state.
	 * @param name
	 * @return@true if the specified CL can transit to the activate state.
	 */
	public boolean canActivate(String name){
		CLInfo baseCL = getCLInfo(name);
		int basePriority = baseCL.getPriority();
		List<String> baseConfResources = baseCL.getConfResources();
		System.out.println("Base priority: "+basePriority);

		for(CLInfo cl: clTable.values()){
			//compare priority first
			if(basePriority > cl.getPriority()){ //competitor has higher priority.

				//Checks whether the competitor is activated now.
				if(cl.getRootComp().mode==Mode.ACTIVE){
					//Checks whether the competitor has same conflict resources.
					List<String> comparedConfResources = cl.getConfResources();
					boolean isContained = false;
					for(String baseResource: baseConfResources){
						if(comparedConfResources.contains(baseResource)){
							isContained = true;
						}
					}
					if(isContained) return false;
				}
			}
		}
		return true;
	}
	

	/**
	 * Prints out information of all control loops.
	 */
	public void printAllCLs(){
		System.out.println("*** Current control loop contents ***");
		for(CLInfo clInfo: clTable.values()){
			clInfo.printValues();
			System.out.println();
		}
	}
	
	public Collection<CLInfo> getAllCLInfo(){
		return clTable.values();
	}
}
