package org.dyncle;

import java.util.ArrayList;

/**
 * Port of component
 * @author h-nakagawa
 * Ver 1.01: add component reference (2011.6.1)
 * @param <T>
 */
@SuppressWarnings("serial")
public class Port<T> extends ArrayList<T>{

	//modes for data store
	public static final int _EXCLUSIVE_MODE_ = 1; //store only the latest one idefaultj
	public static final int _QUEUE_MODE_ = 2;	    //queue type (FIFO)
	public static final int _STACK_MODE_ = 3;	    //stack type 
	
	private int mode = _EXCLUSIVE_MODE_;
	
	private String name;
	private PortType type;
	private Port<T> pairPort;
	private ComponentBehaviour comp; //component that has this port

	public Port(String name, PortType type, ComponentBehaviour aComp){
		this(name, type);
		this.comp = aComp;
	}

	//change data store mode
	public void setMode(int mode){
		this.mode = mode;
	}
	
	public Port(String name, PortType type){
		super();
		this.name = name;
		this.type = type;
	}
	
	public void put(T obj) throws PortException{
		if(pairPort==null) throw new PortException("Pair port is not connected");
		else{
			pairPort.add(obj);
		}
	}

	/**
	 * Gets an object from the port. Acquired object is removed from the port buffer.
	 * @return Object<T>
	 * @throws PortException
	 */
	public T get() throws PortException{
		if(this.isEmpty()) throw new PortException(
				"Tried to get something from the port when it was empty");
		else{
			switch(mode){
			case _EXCLUSIVE_MODE_:
				T data = this.get(this.size()-1);
				this.clear();
				return data;
			case _QUEUE_MODE_:
				return (T) this.remove(0);
			case _STACK_MODE_:
				return (T) this.remove(this.size()-1);
			}
		}
		return null;
	}

	/**
	 * Check whether the port is connected with other port or not.
	 * @return true if the port is connected with other port.
	 */
	public boolean isConnected(){
		return (pairPort!=null);
	}


	/**
	 * Check whether objects are stored in the port buffer.
	 * @return true if the port buffer stores objects.
	 */
	public boolean isArrive(){
		if(this.isEmpty()) return false;
		else return true;
	}
	

	public Port<T> getPairPort() {
		return pairPort;
	}

	public void setPairPort(Port<?> pairPort) {
		this.pairPort = (Port<T>) pairPort;
	}

	public String getName() {
		return name;
	}

	public PortType getType() {
		return type;
	}
	
	
	public ComponentBehaviour getComponent() {
		return comp;
	}

	public void setComponent(ComponentBehaviour comp) {
		this.comp = comp;
	}

	//TODO: get priority from the CLTable (by using parent component's name)
	public int getPriority(){
		return this.comp.getPriority();
	}
}