package org.nak.cleaning;

import java.awt.Point;
import java.lang.reflect.Constructor;

import org.nak.cleaning.VirtualRobot.BatteryEmptyException;
import org.nak.cleaning.VirtualRobot.DustOverloadedException;
import org.nak.cleaning.map.FieldMap;
import org.nak.cleaning.map.SampleMap01;
import org.nak.cleaning.object.Appearance;
import org.nak.cleaning.object.Obstacle;
import org.nak.cleaning.object.OtherRobot;

/**
 * Simulator@class
 * Ver2.00
 * @author h-nakagawa
 *	Ver2.00-API of Virtual Robot is only provided through the simulator interface
 *				(for displaying Exceptions on Simulator GUI by catching them)
 */
public class Simulator {
	private static Simulator instance;
	
	private static String mapPackagePath = "org.nak.cleaner.map";
//	private static String mapPackagePathOther = "org.nak.cleaner.experiment";
	
	//Interval of STOP check
	private static final int _STOP_INTERVAL_= 100;
	
	//Whether load amount should be displayed
	//** We should change "VirtualRobot._LIMIT_OF_GARBAGE_BAG_"
//	public boolean isLoadAmountConsidered = false; //if it is false, GUI always displays "--"
	public boolean isLoadAmountConsidered = true;

	//GUI class for simulator
	SimulatorGUI gui;
	
	FieldMap map;

	//Robot@on simulator
	VirtualRobot robot;
	
	//Whether simulator can be executed. (It becomes true when start button pushed.)
	Boolean isReady = false;
	
	//Whether simulator is suspended. (It becomes true when stop button pushed.)
	Boolean isStoped = false;
	
	/**
	 * The constructor.
	 */
	public Simulator(){
		super();
		
		map = new SampleMap01();
//		setFieldMap("Map4");
		instance = this;
		
		robot = null;
		
		
		//start up a thread for GUI management
		GUIManager guiManager = new GUIManager(this);
		guiManager.start();

		
//		robot = new VirtualRobot(map);
//		robot = (VirtualRobot)(map.map.get(map.initPoint));
	}

	public static Simulator getSimulator(){
		if(instance==null) instance = new Simulator();
		return instance;
	}
	
	/**
	 * Sets a map on simulator.
	 * @param mapName Name of the map
	 */
	public void setFieldMap(String mapName){
		//Use reflection
		try{
		     // Acquire the map class
//			Class<FieldMap> cls = (Class<FieldMap>) Class.forName(mapName);
//			Class cls = Class.forName(mapPackagePath+"."+mapName);
			Class cls = Class.forName(mapName);
//			if(cls==null){
//				cls = Class.forName(mapPackagePathOther+"."+mapName);
//			}
			// Acquire the method (Constructor)
//			Method method = cls.getMethod(mapName, null);
			Constructor cons = cls.getConstructor(null);
			//@Set the arguments
			Object[] args = new Object[]{null};
			// Execute Constructor
			map = (FieldMap) cons.newInstance();
//			Object ret = method.invoke(cls.newInstance(),null);
			
			// Generate robot
				getRobot();
//				robot.setPoint(map.initPoint);
				robot.setPoint();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public FieldMap getFieldMap(){
		return map;
	}

	/**
	 * Returns whether simulator can be executed. (for GUI)
	 * @return
	 */
	public Boolean isReady() {
		return isReady;
	}
	
	/**
	 * Returns whether dust items remain in the field. (for checking termination condition)
	 * @return Returns true if there are no dust items in the field.
	 */
	public boolean isFinished(){
		if(map.isFinished()){
			System.out.println("\n*** All dust items are cleaned up. ***");
			gui.printOut("*** All dust items are cleaned up. ***\n");
			return true;
		}
		else return false;
	}



	public VirtualRobot getRobot() {
		if(robot==null){
			robot = new VirtualRobot(this);
		}
		return robot;
	}

	private boolean isCrashedRobots(){
		if(map.obstacle != null){
//			System.out.println("Robot: "+robot.currentPoint+"\tObstacle: "+map.obstacle.point);
			if(map.obstacle != null && 
					robot.currentPoint.x == map.obstacle.point.x &&
					robot.currentPoint.y == map.obstacle.point.y) return true;
		}
		return false;
	}
	
	public Obstacle getObstacle(){
		return map.obstacle;
	}

	public void setObstacle(Obstacle or){
		map.obstacle = or;
	}
	

/////////////////////////////// API for Robot manipulation ///////////////////////////////
	public Point find(String targetType){
		try {
			return robot.find(targetType);
		} catch (BatteryEmptyException e) {
			batteryEmpty();
		}
		return null;
	}

	public int moveNext(Direction d){
		try {
			int result = robot.moveNext(d);
			if(!isCrashedRobots()) return result;
			else robotsCrashed();
		} catch (BatteryEmptyException e) {
			batteryEmpty();
		}
		return(-1);
	}


	public boolean pickUp(){
		try {
			return robot.pickUp();
		} catch (BatteryEmptyException e) {
			batteryEmpty();
		} catch (DustOverloadedException e) {
			dustOverloaded();
		}
		
		return false;
	}

	public boolean sweep(){
		try {
			return robot.sweep();
		} catch (BatteryEmptyException e) {
			batteryEmpty();
		} catch (DustOverloadedException e) {
			dustOverloaded();
		}
		return false;
	}

	public boolean wipe(){
		try {
			return robot.wipe();
		} catch (BatteryEmptyException e) {
			batteryEmpty();
		} catch (DustOverloadedException e) {
			dustOverloaded();
		}
		return false;
	}
	
	/**
	 * Exception handling for out of charge
	 */
	private void batteryEmpty(){
		gui.printOut("*** Battery is empty ... ***\n");
		isStoped = true;
//		e.printStackTrace();
	}

	/**
	 * Exception handling for overload dust items
	 */
	private void dustOverloaded(){
		gui.printOut("*** Robot overloaded dust items... ***\n");
		isStoped = true;
//		e.printStackTrace();
	}

	/**
	 * Exception handling for crashing obstacles
	 */
	private void robotsCrashed(){
		gui.printOut("*** Robot is crashed ... ***\n");
		isStoped = true;
//		e.printStackTrace();
	}

	
	
	/**
	 * Charges battery.
	 * @return returns true when the robot succeeded charging, otherwise returns false.
	 * (Charge is failed when robot is not in the same point of the station.)
	 */
	public boolean chargeBattery(){
		return robot.chargeBattery();
	}

	/**
	 * Unloads dust items into the dust bin
	 * @return returns true when the robot succeeded unloading, otherwise returns false.
	 * (Unload is failed when robot is not in the same point of the dust bin.)
	 */
	public boolean unloadDust(){
		return robot.unloadDust();
	}
		
	/**
	 * Checks whether there are any dust items at the robot position.
	 * @return returns true if there is a dust item at the robot's position.
	 */
	public boolean existsDust(){
		return map.existsDust(robot.currentPoint);
	}
	
	/**
	 * Checks whether there are any battery stations at the robot position.
	 * @return returns true if there is a battery station at the robot's position.
	 */
	public boolean isLocatedOnStation(){
		return map.isLocated(robot.currentPoint, "ChargeStation");
	}
	
	/**
	 * Checks whether there are any dust bins at the robot position.
	 * @return returns true if there is a dust bin at the robot's position.
	 */
	public boolean isLocatedOnDustBin(){
		return map.isLocated(robot.currentPoint, "DustBin");
	}
	
	/**
	 * Returns the current robot's position.
	 * @return Point where the robot is located.
	 */
	public Point getCurrentPoint() {
		return robot.currentPoint;
	}
	
	/**
	 * Gets appearance of the object that the robot is located in the same place.
	 * @return Appearance of the object
	 */
	public Appearance watchObject(){
		return map.getAppearance(robot.currentPoint);
	}
	
	/**
	 * Gets the current battery level with interval. (for EXP2)
	 * @return current battery level (integer type).
	 */
	public void checkBatteryWithInterval() {
		robot.checkBatteryWithInterval();
	}
	
	/**
	 * Returns whether the battery check is completed. (for EXP2)
	 * @return It returns ture if the battery check is finished.
	 */
	public boolean isBatteryChecked(){
		return robot.isBatteryChecked;
	}
	
	/**
	 * Resets the battery check flag. (for EXP2)
	 */
	public void resetBatteryCheck(){
		robot.isBatteryChecked = false;
	}
	

	/**
	 * Returns the current known battery level. (for EXP2)
	 * visibleBattery is never updated until the checkBatteryWithInterval() method is executed.
	 * @return
	 */
	public int getVisibleBattery(){
		return robot.visibleBattery;
	}

	/**
	 * Gets the current battery level.
	 * @return current battery level (integer type).
	 */
	public int getBattery() {
		return robot.getBattery();
	}

	/**
	 * Gets the current load amount of the robot.
	 * @return current load amount (integer type).
	 */
	public int getLoadAmount() {
		return robot.getLoadAmount();
	}
	
	/**
	 * Checks whether the camera module of the robot is broken. 
	 * @return returns true if the camera module of the robot is broken.
	 */
	public boolean isCameraBroken() {
		return robot.isCameraBroken;
	}
	

///////////////////////////////// UTILITIES for GUI /////////////////////////////////

	protected void println(String str){
		gui.printOut(str+"\n");
	}

	protected void moveRobot(Point targetP, Direction direction){
		while(isStoped){
			try {
				Thread.sleep(_STOP_INTERVAL_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
		gui.moveRobot(targetP, direction);
	}

	/** Redraw GUI  */
   protected void guiClear() {
		while(isStoped){
			try {
				Thread.sleep(_STOP_INTERVAL_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		gui.clear();
  }
   
   protected void batteryLabelChange(int bLevel) {
		while(isStoped){
			try {
				Thread.sleep(_STOP_INTERVAL_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		gui.changeBatteryLabel(bLevel);
	}
   
   protected void loadAmountChange(int loadAmount) {
		while(isStoped){
			try {
				Thread.sleep(_STOP_INTERVAL_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		gui.changeLoadAmounLabel(loadAmount);
	}
}
