package org.nak.cleaning;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.swt.widgets.Text;
import org.nak.cleaning.object.ChargeStation;
import org.nak.cleaning.object.DustBin;
import org.nak.cleaning.object.Element;
import org.nak.cleaning.object.EmptyCan;
import org.nak.cleaning.object.Leaves;
import org.nak.cleaning.object.Litter;
import org.nak.cleaning.object.Obstacle;
import org.nak.cleaning.object.PileOfJunk;
import org.nak.cleaning.object.Puddle;
import org.nak.cleaning.object.Sand;

/**
 * GUI for Simulator
 * @version 1.2 otherRobot -> Obstacle
 * 	delete updateThread
 * @author h-nakagawa
 *
 */
public class SimulatorGUI implements Runnable{
	
	Simulator simulator;
	Display display;
//	Thread updateThread;  //  @jve:decl-index=0:
	boolean isMapRead = false;
	//isMapChanged: becomes true when the map is read or dust items are cleaned.
	boolean isRobotMoving = false;
	
	//Variables for injecting or moving objects
	String injectedItem; //Item to be injected
	boolean isInjectButtonSelected = false; 
	boolean isMoveItemButtonSelected = false; 
	boolean isObstacleLocated = false; //Whether obstacles are located in the map.
	//Current simulator considers only one obstacle in the map.
	java.awt.Point selectedCell=null; //Cell where cursor is currently located.
	java.awt.Point mouseDownCell=null; //Cell where cursor was located when the mouse is pushed down.
	Element moveTargetItem=null; //Item to be moved

	//Animation speed [ms] (=interval time after moving)
//	static final int _DEFAULT_ANIMATION_INTERVAL_ = 150;
	static final int _DEFAULT_ANIMATION_INTERVAL_ = 200;
//	int animationSpeed = 250;
	int animationSpeed = _DEFAULT_ANIMATION_INTERVAL_;
	
	//Maps to be read
//	String mapName = "org.nak.cleaning.experiment.Map22"; //for JAWS EXP  //  @jve:decl-index=0:
//	String mapName = "org.nak.cleaning.experiment.Map13"; //for IWMST EXP  //  @jve:decl-index=0:
//	String mapName = "org.nak.cleaning.experiment.Map54"; //for SEAMS EXP (with dustbin) //  @jve:decl-index=0:
//	String mapName = "org.nak.cleaning.experiment.Map55"; //for SEAMS demo and Dr. thesis EXP (with dustbin) //  @jve:decl-index=0:
	String mapName = "org.nak.cleaning.experiment.Map77"; //for EASy demo  //  @jve:decl-index=0:
	int mapWidth=0;
	int mapDepth=0;

	protected Image myRobot;
	protected Image myChargeStation;
	protected Image myDustBin;
	protected Image myEmptyCan;
	protected Image myObstacle;
	protected Image myLeaves;
	protected Image myLitter;
	protected Image myPileOfJunk;
	protected Image mySand;
	protected Point myRobotPos = new Point(0,0);  //  @jve:decl-index=0:
//	protected OtherRobot otherRobot;  //  @jve:decl-index=0:
//	protected enum Direction {STOP, LEFT, RIGHT, UP, DOWN}
	protected Direction myDirection = Direction.STOP;  //  @jve:decl-index=0:
//	String [] items = {"charge station","dust bin","other robot", "empty can","leaves","litter","pile of junk","puddle","sand"};
	String [] items = {"Charge station","Dust bin","Obstacle", "Empty can","Leaves","Litter","Pile of junk","Puddle","Sand"};

	
	public SimulatorGUI(Simulator sim){
//		public SimulatorGUI(Simulator sim, Display disp){
		super();
		display = new Display();
		simulator = sim;
		simulator.gui = this;
//		otherRobot = simulator.otherRobot;
//		display = disp;
		setUp();

		createSShell();
      sShell.open();
      createCanvas(); //2012.5.24 Draw canvas after opening the shell

      while (!sShell.isDisposed()) {
         if (!display.readAndDispatch())
            display.sleep();
      }
      System.out.println("terminating...");
      display.dispose();
	}

//	private int count = 0;
	private int _BLOCK_SIZE_ = 50;

	public Shell sShell = null;  //  @jve:decl-index=0:visual-constraint="10,10"
	private Canvas canvas = null;
	private Composite composite = null;
	private Label label = null;
	private Label[] x_labels = null;
	private Label[] y_labels = null;
	private Button batteryBrokenButton = null;
	private Button startButton = null;
	private Button readMapButton = null;
	private Label batteryLevLabel = null;
	private Label batteryLevValueLabel = null;
	private Label loadAmountLabel = null;
	private Label loadAmountValueLabel = null;
	private Button stopButton = null;
	private Text console = null;
	private Button cameraBrokenButton = null;
	private Combo itemCombo = null;
	private Label injectItemLabel = null;
	private Button injectButton = null;
	private Label speedLabel = null;
	private Slider speedSlider = null;
	private Button fileDialogButton = null;
	private Label mapNameLabel = null;
	private Text mapNameText = null;
	private Button moveItemButton = null;

	private void setUp(){
		//read images
	   myRobot = new Image(display, "image/robot-trans_50.gif");
	   myEmptyCan = new Image(display, "image/can-trans_50.gif");
	   myChargeStation = new Image(display, "image/chargeSt-trans_50.gif");
	   myDustBin = new Image(display, "image/dustBin_50.gif"); 
	   //DustBin can't be transparent by constraints of the image
	   myObstacle = new Image(display, "image/obstacle-trans_50.gif");
	   myLeaves = new Image(display, "image/leaves-trans_50.gif");
	   myLitter = new Image(display, "image/litter-trans_50.gif");
	   myPileOfJunk = new Image(display, "image/pileOfJunk-trans_50.gif");
	   mySand = new Image(display, "image/sand-trans_50.gif");

	}
	
	/**
	 * This method initializes sShell
	 */
	public void createSShell() {
		sShell = new Shell();
		sShell.setText("Cleaning Simulator");
		sShell.setSize(new Point(803, 686));
		sShell.setLayout(null);
		createComposite();
		console = new Text(sShell, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		console.setBounds(new Rectangle(17, 496, 648, 115));
		sShell.pack();
	}

	/**
	 * This method initializes canvas	
	 *
	 */
	private void createCanvas() {
		canvas = new Canvas(composite, SWT.NO_BACKGROUND);
//		canvas = new Canvas(composite, SWT.NONE);
		canvas.setBounds(new Rectangle(179, 66, 451, 385));
		
		canvas.addPaintListener(new PaintListener() {
			//paintControl: same as OnDraw(CDC* pDC) in MFC
			public void paintControl(PaintEvent e) {

				if(isMapRead){
					//Create Image covering canvas (for drawing map)
					Point canvasSize = canvas.getSize();
//	            final Image image = new Image(display, canvasSize.x, canvasSize.y);
					Image image = new Image(display, canvasSize.x, canvasSize.y);
	            GC gc1 = new GC(image);

					//draw lines
					for (int i=0;i<=mapWidth;i++){
						gc1.drawLine(i*_BLOCK_SIZE_, 0, i*_BLOCK_SIZE_, mapDepth*_BLOCK_SIZE_);
					}
					for (int j=0;j<=mapDepth;j++){
						gc1.drawLine(0, j*_BLOCK_SIZE_, mapWidth*_BLOCK_SIZE_, j*_BLOCK_SIZE_);
					}

					//Change cell's color if the cell is selected
					if((isInjectButtonSelected || isMoveItemButtonSelected) && selectedCell!=null){
						if(0<=selectedCell.x && 0<=selectedCell.y){
							Color blue= display.getSystemColor(SWT.COLOR_BLUE);
							gc1.setBackground(blue);
							gc1.fillRectangle(selectedCell.x*_BLOCK_SIZE_, selectedCell.y*_BLOCK_SIZE_, _BLOCK_SIZE_, _BLOCK_SIZE_);
			            blue.dispose();
						}
					}
//					Color red = display.getSystemColor(SWT.COLOR_RED);
//	            Color blue= display.getSystemColor(SWT.COLOR_BLUE);
	            
	            //1st object
//	            gc1.setBackground(red);
//	            gc1.fillRectangle(10,10,80,80);
	            
	            //2nd object
//	            gc1.setForeground(blue);
//	            gc1.setBackground(blue);
//	            gc1.fillOval(110,10,80,80);              

					Element elem;
					for(int i=0; i<mapWidth;i++){
						for(int j=0; j<mapDepth; j++){
							java.awt.Point currentPoint = new java.awt.Point(i,j);
							elem = simulator.getFieldMap().getHashMap().get(currentPoint);
							if(elem!=null){
								Point objectPos;
								switch(elem.type){
								case CHARGE_STATION:
//									System.out.println(currentPoint+" is located a charge station.");
									objectPos = getOriginPosition(currentPoint, myChargeStation);
									gc1.drawImage(myChargeStation, objectPos.x, objectPos.y);
									break;
								case DUST_BIN:
//									System.out.println(currentPoint+" is located a dust bin.");
									objectPos = getOriginPosition(currentPoint, myDustBin);
									gc1.drawImage(myDustBin, objectPos.x, objectPos.y);
									break;
								case EMPTY_CAN:
//									System.out.println(currentPoint+" is located an empty can.");
									objectPos = getOriginPosition(currentPoint, myEmptyCan);
									gc1.drawImage(myEmptyCan, objectPos.x, objectPos.y);
									break;
								case LEAVES:
//									System.out.println(currentPoint+" is located leaves.");
									objectPos = getOriginPosition(currentPoint, myLeaves);
									gc1.drawImage(myLeaves, objectPos.x, objectPos.y);
									break;
								case LITTER:
//									System.out.println(currentPoint+" is located litter.");
									objectPos = getOriginPosition(currentPoint, myLitter);
									gc1.drawImage(myLitter, objectPos.x, objectPos.y);
									break;
								case PILE_OF_JUNK:
//									System.out.println(currentPoint+" is located pile of junk.");
									objectPos = getOriginPosition(currentPoint, myPileOfJunk);
									gc1.drawImage(myPileOfJunk, objectPos.x, objectPos.y);
									break;
								case SAND:
//									System.out.println(currentPoint+" is located pile of sand.");
									objectPos = getOriginPosition(currentPoint, mySand);
									gc1.drawImage(mySand, objectPos.x, objectPos.y);
									break;
								case ROBOT:
//									System.out.println(currentPoint+" is located a cleaning robot.");
									break;
								default:
									System.err.println(currentPoint+" is located "+elem.type+".");
								}
							}
						}
					}

					//Draw robot
					gc1.drawImage(myRobot, myRobotPos.x, myRobotPos.y);
					//Draw obstacle
					if(isObstacleLocated){
						Point objectPos = getOriginPosition(simulator.getObstacle().point, myObstacle);
						gc1.drawImage(myObstacle, objectPos.x, objectPos.y);
					}

//					myRobotPos.x += 10;
//				   myRobotPos.y -= 10;
//				   System.out.println("(x, y) = "+myRobotPos.x + " "+myRobotPos.y);


	            gc1.dispose();
//	            red.dispose();
//	            blue.dispose();

	            e.gc.drawImage(image, 0, 0);
					// gc.dispose() is executed by Eclipse Framework
				}
			}
		});
		canvas.addMouseMoveListener(new org.eclipse.swt.events.MouseMoveListener() {
			public void mouseMove(org.eclipse.swt.events.MouseEvent e) {
				// Cursor is moving
				if(isInjectButtonSelected || isMoveItemButtonSelected){
					java.awt.Point tmpCell = calculateContainingCell(new Point(e.x,e.y));
//					System.out.println("mouseMove()"+e+"\t on "+tmpCell); 
					if(!tmpCell.equals(selectedCell)){
						selectedCell = tmpCell;
						display.asyncExec(new Runnable() {
							public void run() {
								canvas.redraw();
								canvas.update();
//								canvas.layout();
							}
						});
					}
				}
			}
		});
		canvas.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {   
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
//				System.out.println("mouseDown()"); 
				// Mouse button is pushed down (store the point)
				if(e.button==1){
					mouseDownCell= new java.awt.Point(calculateContainingCell(new Point(e.x,e.y)));
//					System.out.println("\tmouseDownCell: "+mouseDownCell);
//					mouseDownCell= new java.awt.Point(selectedCell.x, selectedCell.y);
					if(isMoveItemButtonSelected){
						//Identify the item that should be moved
						moveTargetItem = simulator.map.map.get(mouseDownCell);
						Obstacle obstacle = simulator.getObstacle();
						if(obstacle!=null && mouseDownCell.x == obstacle.point.x &&
								mouseDownCell.y == obstacle.point.y){
							moveTargetItem = obstacle;
						}
						if(moveTargetItem!=null){
							System.out.println("\tTargetItem: "+moveTargetItem.toString());
						}
					}
				}
			}
			public void mouseUp(org.eclipse.swt.events.MouseEvent e) {    
//				System.out.println("mouseUp()"); 
				// Mouse button is released (Compare points, Inject items, or clear the task)
				if(e.button==1){
					if(mouseDownCell.equals(selectedCell)){
//					if(selectedCell.equals(mouseDownCell)){
						if(isInjectButtonSelected){//in the case of injecting items
							System.out.println(injectedItem+" is thown at "+selectedCell);

							Element obj;
							java.awt.Point p = new java.awt.Point(selectedCell.x, selectedCell.y);
							if(injectedItem.equals("Charge station")){
								simulator.map.map.put(p, new ChargeStation());
							}
							else{
								if(injectedItem.equals("Dust bin")){
									simulator.map.map.put(p, new DustBin());
								}
								else{
									if(injectedItem.equals("Empty can")){
										simulator.map.map.put(p, new EmptyCan());
									}
									else{
										if(injectedItem.equals("Leaves")){
											simulator.map.map.put(p, new Leaves());
										}
										else{
											if(injectedItem.equals("Litter")){
												simulator.map.map.put(p, new Litter());
											}
											else{
												if(injectedItem.equals("Pile of junk")){
													simulator.map.map.put(p, new PileOfJunk());
												}
												else{
													if(injectedItem.equals("Puddle")){
														simulator.map.map.put(p, new Puddle());
													}
													else{
														if(injectedItem.equals("Sand")){
															simulator.map.map.put(p, new Sand());
														}
														else{
															if(injectedItem.equals("Obstacle")){
																simulator.setObstacle(new Obstacle(p));
																isObstacleLocated = true;
																System.out.println("Obstacle appears at "+simulator.getObstacle().point);
//																simulator.map.map.put(p, new Sand());
															}
														}
													}
												}
											}
										}
									}
								}
							}
							//reset variables
							isInjectButtonSelected = false;
							selectedCell=null;
							mouseDownCell=null;
						}
					}
					else{
						if(isMoveItemButtonSelected){//in the case of moving items
							System.out.println(moveTargetItem+" is moved at "+selectedCell);

							//Obstacle is excepted because the map does not store the information about it.
							if(moveTargetItem==simulator.getObstacle()){
								//Other robot cleans dust items before moving. (in the previous version)
//								Element object = simulator.map.map.get(mouseDownCell);
//								if(object!=null && object instanceof Dust){
//									System.out.println("Other robot cleaned "+
//											((Dust)object).type+" at "+mouseDownCell);
//									simulator.map.map.remove(mouseDownCell);
//								}
								simulator.getObstacle().point.x = selectedCell.x;
								simulator.getObstacle().point.y = selectedCell.y;
							}
							else{
								simulator.map.map.remove(mouseDownCell);
								java.awt.Point p = new java.awt.Point(selectedCell.x, selectedCell.y);
								simulator.map.map.put(p, moveTargetItem);
							}


							//reset variables
							moveTargetItem = null;
							isMoveItemButtonSelected = false;
							selectedCell=null;
							mouseDownCell=null;
						}
					}

					display.asyncExec(new Runnable() {
						public void run() {
							canvas.redraw();
//							canvas.update();
						}
					});

				}
			}
		});
	
	}
	
	
	/**
	 * Calculates the left upper point (swt.graphics.Point) for drawing images.
	 * Note that input point is java.awt.Point representing the cell in the map,
	 * and out put is swt.graphics.Point for drawing.
	 * @param p: java.awt.Point in the map 
	 * @param img image to be drawn
	 * @return swt.graphics.Point for drawing image. (Corresponding to the left upper point of the cell.)
	 */
	private Point getOriginPosition(java.awt.Point p, Image img){
		Point centerPoint = getCenterPosition(p);
		return(new Point(centerPoint.x - img.getBounds().width/2, centerPoint.y - img.getBounds().height/2));
	}

	/**
	 * Translates given coordination (java.awt.Point) in the map 
	 * into the Point for drawing (swt.graphics.Point), and returns the central point.
	 * @param p Point (java.awt.Point) in the map.
	 */
	private Point getCenterPosition(java.awt.Point p){
		return(getCenterPosition(p.x, p.y));
	}

	
	/**
	 * Calculates the central point (swt.graphics.Point) for drawing
	 * from the given Point representing the cell in the map.
	 * @param x
	 * @param y
	 */
	private Point getCenterPosition(int x, int y){
		return(new Point((2*x+1)*_BLOCK_SIZE_/2, (2*y+1)*_BLOCK_SIZE_/2));
	}

	/**
	 * Calculates the cell in the map from the given swt.graphics.Point in GUI.
	 * Note that the given point, including (0, 0), is out of map, returns (-1, -1).
	 * @param e swt.graphics.Point in GUI
	 * @return cell (java.awt.Point) in the map.
	 */
	private java.awt.Point calculateContainingCell(Point e){
		if(0<e.x && 0<e.y){
			int x = (int)((e.x)/_BLOCK_SIZE_);
			int y = (int)((e.y)/_BLOCK_SIZE_);
			if(x<mapWidth && y<mapDepth){
				return(new java.awt.Point(x, y));
			}
		}
		return(new java.awt.Point(-1, -1));
	}

	// This method is not used now. (from @IT)
   /** Redraws at intervals */
   public void run() {
   	try {
   		display = canvas.getDisplay();
   		while (true) {
   				// make display execute canvas.redraw()
   				display.asyncExec(new Runnable() {
   					public void run() {
   						if(!canvas.isDisposed()) canvas.redraw();
   					}
   				});
   			Thread.sleep(50000);
   			synchronized(this){
   			}
   		}
   	} catch (InterruptedException ignore) { // terminate if it receives interrupt.
      }
  }

   /** Prints out the string. */
   public void printOut(final String str) {
		display.asyncExec(new Runnable() {
			public void run() {
		   	console.append(str);
			}
		});
  }

   public void clear() {
		display.asyncExec(new Runnable() {
			public void run() {
				canvas.redraw();
				canvas.update();
			}
		});
  }
   


   /** Moves the robot to the specified point. */
   public void moveRobot(final java.awt.Point targetP, final Direction direction) {
   	Point targetSWTPoint = getOriginPosition(targetP, myRobot);
//   	printOut("target: "+targetP+"\t real origin point: "+targetSWTPoint);
   	try {
   		myDirection = direction;
   		while (myRobotPos.x != targetSWTPoint.x || myRobotPos.y != targetSWTPoint.y) {
				// make display execute canvas.redraw()
				display.asyncExec(new Runnable() {
					public void run() {
						if(!canvas.isDisposed()){
							switch (myDirection) {
							//TODO: change NORTH and SOUTH in the future.
							case NORTH:
							   myRobotPos.y += 10;
								break;
							case SOUTH:
							   myRobotPos.y -= 10;
								break;
							case EAST:
							   myRobotPos.x += 10;
								break;
							case WEST:
							   myRobotPos.x -= 10;
								break;

							default:
								break;
							}
//						   System.out.println("(x, y) = "+myRobotPos.x + " "+myRobotPos.y);
							canvas.redraw();
						}
					}
				});
				Thread.sleep(animationSpeed);
   		}
   		myDirection = direction.STOP;
   	} catch (InterruptedException ignore) { // terminate if it receives interrupt.
      }
   }

   /** Updates the batteryLabel */
   public void changeBatteryLabel(final int bLevel) {
		display.asyncExec(new Runnable() {
			public void run() {
				if(!batteryLevValueLabel.isDisposed()){
					batteryLevValueLabel.setText(Integer.toString(bLevel));
					batteryLevValueLabel.redraw();
				}
			}
		});
   }

   /** Updates the loadAmounLabel */
   public void changeLoadAmounLabel(final int loadAmount) {
		display.asyncExec(new Runnable() {
			public void run() {
				if(!loadAmountValueLabel.isDisposed()){
					if(simulator.isLoadAmountConsidered){
						loadAmountValueLabel.setText(Integer.toString(loadAmount));
						loadAmountValueLabel.redraw();
					}
				}
			}
		});
   }


   /** Updates the Map text box */
   private void changeMapText(final String mapString) {
		display.asyncExec(new Runnable() {
			public void run() {
				if(!mapNameText.isDisposed()){
					mapNameText.setText(mapString);
					mapNameText.redraw();
				}
			}
		});
   }

	/**
	 * This method initializes composite	
	 *
	 */
	private void createComposite() {
		composite = new Composite(sShell, SWT.NONE);
		composite.setLayout(null);
		composite.setBounds(new Rectangle(16, 15, 670, 482));
		label = new Label(composite, SWT.NONE);
		label.setFont(new Font(Display.getDefault(), "Meiryo", 12, SWT.ITALIC));
		label.setBounds(new Rectangle(14, 10, 146, 24));
		label.setText("Cleaning Simulator");
//		createCanvas(); //Move to after the shell opened (2012.5.24)
		
		//Exclude the following buttons from the window (2011.8.28)
		batteryBrokenButton = new Button(composite, SWT.NONE);
//		batteryBrokenButton.setBounds(new Rectangle(19, 171, 109, 27));
		batteryBrokenButton.setText("Battery Broken ");
		cameraBrokenButton = new Button(composite, SWT.NONE);
//		cameraBrokenButton.setBounds(new Rectangle(19, 209, 108, 28));
		cameraBrokenButton.setText("Camera Broken ");

		startButton = new Button(composite, SWT.NONE);
		startButton.setBounds(new Rectangle(19, 98, 106, 28));
		startButton.setText("Start");
		readMapButton = new Button(composite, SWT.NONE);
		readMapButton.setBounds(new Rectangle(19, 53, 108, 28));
		readMapButton.setText("Read Map");
		batteryLevLabel = new Label(composite, SWT.NONE);
		batteryLevLabel.setBounds(new Rectangle(17, 398, 95, 18));
//		Font defaultFont = batteryLevLabel.getFont();
		Font font = new Font(display, "",11,SWT.BOLD);
		batteryLevLabel.setFont(font);
		batteryLevLabel.setText("Battery level: ");
		batteryLevValueLabel = new Label(composite, SWT.NONE);
		batteryLevValueLabel.setBounds(new Rectangle(115, 398, 30, 18));
		batteryLevValueLabel.setFont(font);
		batteryLevValueLabel.setAlignment(SWT.RIGHT);

		loadAmountLabel = new Label(composite, SWT.NONE);
		loadAmountLabel.setBounds(new Rectangle(17, 425, 95, 18));
		loadAmountLabel.setFont(font);
		loadAmountLabel.setText("Load amount: ");
		loadAmountValueLabel = new Label(composite, SWT.NONE);
		loadAmountValueLabel.setBounds(new Rectangle(115, 425, 30, 18));
		loadAmountValueLabel.setFont(font);
		loadAmountValueLabel.setAlignment(SWT.RIGHT);
		if(!simulator.isLoadAmountConsidered) loadAmountValueLabel.setText("--");
		stopButton = new Button(composite, SWT.NONE);
		stopButton.setBounds(new Rectangle(19, 134, 104, 28));
		stopButton.setText("Stop");
		cameraBrokenButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						System.out.println("Camera is broken."); 
						// Camera is broken
						simulator.robot.isCameraBroken = true;
					}
				});
		createDustItemCombo();
		injectItemLabel = new Label(composite, SWT.NONE);
		injectItemLabel.setBounds(new Rectangle(18, 246, 116, 20));
		injectItemLabel.setText("Inject Item: ");
		injectButton = new Button(composite, SWT.NONE);
		injectButton.setText("Inject");
		injectButton.setSize(new Point(108, 28));
		injectButton.setLocation(new Point(19, 300));
		speedLabel = new Label(composite, SWT.NONE);
		speedLabel.setBounds(new Rectangle(18, 178, 45, 20));
		speedLabel.setText("Speed:");
		speedSlider = new Slider(composite, SWT.NONE);
		speedSlider.setBounds(new Rectangle(19, 197, 122, 16));
		speedSlider.setSelection(200);
		speedSlider.setIncrement(10);
		speedSlider.setMinimum(20);
		speedSlider.setMaximum(510);
		fileDialogButton = new Button(composite, SWT.NONE);
		fileDialogButton.setBounds(new Rectangle(538, 12, 91, 24));
		fileDialogButton.setText("Reference");
		mapNameLabel = new Label(composite, SWT.NONE);
		mapNameLabel.setBounds(new Rectangle(182, 13, 40, 22));
		mapNameLabel.setText("Map:");
//		mapNameLabel.setText("Map: "+mapName);
		mapNameText = new Text(composite, SWT.BORDER);
		mapNameText.setBounds(new Rectangle(223, 11, 307, 22));
		mapNameText.setText(mapName);
		moveItemButton = new Button(composite, SWT.NONE);
		moveItemButton.setText("Move Item");
		moveItemButton.setSize(new Point(108, 28));
		moveItemButton.setLocation(new Point(19, 338));
		moveItemButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						System.out.println("Move Item button was selected"); 
						isMoveItemButtonSelected = true;
					}
				});
		fileDialogButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						// Set dialog
						//String path = "."; // default directory
						String currentPath = new File(".").getAbsoluteFile().getParent();
//						System.out.println("currentPath: " + currentPath);
						String[] ext = {"*.class"}; //extension (suffix) to be displayed
						//String[] ext = {"*.html;*.css","*.java"};

                  FileDialog dialog = new FileDialog(sShell, SWT.OPEN);

                  // set default directory
                  dialog.setFilterPath(currentPath);
                  // set extension filter
                  dialog.setFilterExtensions(ext);

                  if(dialog.open()!=null){
                     String relativeDir =  dialog.getFilterPath();
                     String relativeFile =  dialog.getFileName();
                     System.out.println("Selected file: " + relativeFile);
                     int index = relativeDir.indexOf(currentPath);
                     
                     if(index==0){
                        //TODO: current code depends on the eclipse execution.
                     	 relativeDir = relativeDir.substring(currentPath.length()+1);
                     	 if(relativeDir.startsWith("bin\\")){ //for eclipse
                     		 relativeDir = relativeDir.substring(4);
                     	 }
                     	 // change delimiter from "\" to "." 
                     	 relativeDir = relativeDir.replace('\\', '.');

                     	 // delete ".class"
                     	 if(relativeFile.endsWith(".class")){
                     		 relativeFile = relativeFile.substring(0, relativeFile.length()-6);
                     	 }

                     	 mapName = relativeDir + "." + relativeFile;
//                         return relativeDir + "/" + dialog.getFileName();
                     	 
                         changeMapText(mapName);


                     }else{
                     	System.err.println("You must select a class file in project work directory.");
//                     	MessageDialog md = new MessageDialog();
//                     	MessageDialog.openWarning(sShell, //"LG3D WRPE Plug-in",
//                     			"Warning",
//                              "You must select a class file in project work directory.");
                     }
                  }
					}
				});
		speedSlider.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
//				System.out.println("selected Point"+speedSlider.getSelection());
				animationSpeed = (int)(_DEFAULT_ANIMATION_INTERVAL_*100/speedSlider.getSelection());
			}
		});
		injectButton.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
//				System.out.println("Inject button was selected"); 
				isInjectButtonSelected = true;
				// TODO Auto-generated Event stub widgetSelected()
			}
		});
		stopButton.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public synchronized void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				// suspend the simulator
				simulator.isStoped = true;
				
			}
		});
		readMapButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						mapName = mapNameText.getText();
						System.out.println("read map: "+mapName); 
						simulator.setFieldMap(mapName); //read specified map file
//		            map = simulator.map;
		            mapWidth = simulator.map._WIDTH_;
						mapDepth = simulator.map._DEPTH_;
						System.out.println("Robot position: "+simulator.robot.currentPoint);
						myRobotPos = getOriginPosition(simulator.robot.currentPoint,myRobot);
						
						isMapRead = true;


						//set robot's initial place
						java.awt.Point robotP = simulator.getCurrentPoint();
						myRobotPos = getOriginPosition(robotP, myRobot);

						//Redraw (it may not be needed)
	               display.asyncExec(new Runnable() {
	               	public void run() {
	               		if(!canvas.isDisposed()) canvas.redraw();
	               	}});

						//Draw numbers of coordinate
						Point canvasOrigin = canvas.getLocation();
						x_labels = new Label[mapWidth];
						y_labels = new Label[mapDepth];
						
						for(int i=0; i<mapWidth; i++){
							x_labels[i] = new Label(composite, SWT.NONE);
							x_labels[i].setText(Integer.toString(i));
							x_labels[i].pack();
							Rectangle labelSize = x_labels[i].getBounds();
							x_labels[i].setBounds(canvasOrigin.x+((2*i+1)*_BLOCK_SIZE_/2)-labelSize.width/2, 
									canvasOrigin.y-(int)(labelSize.height*1.5), labelSize.width, labelSize.height);
						}

						for(int j=0; j<mapDepth; j++){
							y_labels[j] = new Label(composite, SWT.NONE);
							y_labels[j].setText(Integer.toString(j));
							y_labels[j].pack();
							Rectangle labelSize = y_labels[j].getBounds();
							y_labels[j].setBounds(canvasOrigin.x-(int)(labelSize.width*2.5), 
									canvasOrigin.y+((2*j+1)*_BLOCK_SIZE_/2)-labelSize.height/2, 
									labelSize.width, labelSize.height);
						}

					}
				});
		startButton.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public synchronized void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
//				System.out.println("startSelected()"); 

				simulator.isStoped = false;
				simulator.isReady = true;

			}
		});
		batteryBrokenButton.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
//				System.out.println("widgetSelected()"); 
				//sudden decrease in charge
				simulator.robot.setBattery(VirtualRobot._LOW_LEVEL_);
			}
		});
	}

	
	/**
	 * This method initializes dustItemCombo	
	 *
	 */
	private void createDustItemCombo() {
		itemCombo = new Combo(composite, SWT.NONE);
		itemCombo.setBounds(new Rectangle(17, 263, 107, 23));
		itemCombo.setItems(items); 
		itemCombo
				.addSelectionListener(new org.eclipse.swt.events.SelectionListener() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						injectedItem = itemCombo.getText();
//						System.out.println("widgetSelected()\t"+injectedItem); 
					}
					public void widgetDefaultSelected(
							org.eclipse.swt.events.SelectionEvent e) {
					}
				});
	}


}
