package org.nak.cleaning;

import java.awt.Point;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;

import org.nak.cleaning.object.Element;

/**
 * Robot in the simulator
 * @version 2.00
 * @author h-nakagawa
 * Ver1.1- Add attributes about load amount
 *	Ver2.00-API of Virtual Robot is only provided through the simulator interface
 *				(for displaying Exceptions on Simulator GUI by catching them)
 *
 */
public class VirtualRobot extends Element{
	
	Simulator simulator;

//	enum Switch{ON, OFF}

	public static final int _MAX_BATTERY_ = 100;
	public static final int _MAX_LOAD_AMOUNT_ = 100;
	
	protected static final int _LOW_LEVEL_ = (int)(_MAX_BATTERY_*0.25);
	
	//whether the camera module of the robot is broken or not. (Camera is broken by pushing a button on GUI)
	protected Boolean isCameraBroken = false;


	
	//The maximum of load amount 
	//If we set as 0, it means there are no limit.
	//Otherwise, when the load amount rises to the maximum value, the robot halts.
	//**If we change this value from 0 to other, or to 0, we should change the value of Simulator.isLoadAmountConsidered
	private static final int _LIMIT_OF_GARBAGE_BAG_ = 200;
//	private static final int _LIMIT_OF_GARBAGE_BAG_ = 0;

	//Time to be required (for determining simulator waiting time)
	private static final int _WAIT_FOR_CHARGING_ = 700; //Required time for charging
	private static final int _WAIT_FOR_UNLOADING_ = 700; //Required time for unloading

	
	//// Power consumption for each action  ////
	
	// For Dr. thesis 
	//(Search is not consumed energy because of obstacle checking.
	// On the other hand, cleaning requires higher energy.)  (2011.08.30)
	private static final int _POWER_FOR_MOVE_ = 4;
	private static final int _POWER_FOR_SEARCH_ = 0;
	private static final int _POWER_FOR_PICK_UP_ = 4;
	private static final int _POWER_FOR_SWEEP_ = 4;
	private static final int _POWER_FOR_WIPE_ = 4;

	// Before Dr. thesis (Exp by students, SEAMS11, and others)
	// Battery is consumed by object search
//	private static final int _POWER_FOR_MOVE_ = 4;
//	private static final int _POWER_FOR_SEARCH_ = 3;
//	private static final int _POWER_FOR_PICK_UP_ = 2;
//	private static final int _POWER_FOR_SWEEP_ = 2;
//	private static final int _POWER_FOR_WIPE_ = 2;

	// Exp for PerAda
//	private static final int _POWER_FOR_MOVE_ = 4;
//	private static final int _POWER_FOR_SEARCH_ = 3;
//	private static final int _POWER_FOR_PICK_UP_ = 0;
//	private static final int _POWER_FOR_SWEEP_ = 0;
//	private static final int _POWER_FOR_WIPE_ = 0;

	

	//Current robot located point
	Point currentPoint;

	//Current battery level
	private int battery;

	//Current load amount
	private int loadAmount;

///////////////////////// For EXP2 (currently not used) ///////////////////////////////
	
	//Current displayed battery level (For EXP2)
	// This value may be not same as current battery level.
	// This value is updated by executing checkBatteryWithInterval()
	public int visibleBattery;

	//Whether battery check is complated or not (For EXP2)
	public boolean isBatteryChecked=false;

	//Thread for checking battery level
	private BatteryThread bt;
	//Serial no. of threads (For EXP2)
	private int serialID=0;
	
	//Required time (For EXP2)
	private static final int _WAIT_FOR_MOVE_ = 10;
	private static final int _WAIT_FOR_CLEANING_ = 10;
	
	private static final int _WAIT_FOR_CHECK_BATTERY_ = 1;
//	private static final int _WAIT_FOR_CHECK_BATTERY_ = 10;

////////////////////////////////////////////////////////////////////////////

	

	public VirtualRobot(Simulator sim) {
		super();
		simulator = sim;
		if(sim.map==null){
			System.err.println("VirtualRobot.java: Map has not been set on the simulator.");
		}
//		else{
//			floorMap = sim.map;
//		}
		currentPoint = sim.map.initRobotPoint;
//		setBattery(_MAX_BATTERY_);
		this.battery = _MAX_BATTERY_;
		this.visibleBattery = this.battery;
		this.loadAmount=0;
//		simulator = Simulator.getMapCoordinator();
//		floorMap = simulator.getFieldMap();
	}
	

////////////////////////////////////////////////////////////
///////////////////////   APIs   ///////////////////////////
///////////////////////////////////////////////////////////

	

	/**
	 * Checks whether the specified point is in the map.
	 * @param p Point
	 * @return returns if Point p is in the map, otherwise returns false.
	 */
	public boolean isContainedInMap(Point p){
		return simulator.map.isContainedInMap(p);
	}
	




	/**
	 * Robot sweeps at the current place.
	 * @return returns true if the cleaning by sweeping is succeeded, otherwise returns false.
	 */
	public boolean sweep()throws BatteryEmptyException, DustOverloadedException{
		Element dust;
		reduceBattery(_POWER_FOR_SWEEP_); //energy consumption

		try {
			Thread.sleep(simulator.gui.animationSpeed*2); //required time for cleaning
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		dust = simulator.map.sweep(currentPoint);
		if(dust!=null){
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was swept.");
			simulator.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was swept.");
			simulator.guiClear();
			addDustIntoBag(dust.volume); //add loaded dust items
		}
		else{
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") can not be swept.");
		}
		return dust!=null;
	}
	
	/**
	 * Robot picks up the object at the current place for cleaning
	 * @return@returns true if the cleaning by picking up is succeeded, otherwise returns false.
	 */
	public boolean pickUp()throws BatteryEmptyException, DustOverloadedException{
		Element dust;
			reduceBattery(_POWER_FOR_PICK_UP_); //energy consumption
		try {
			Thread.sleep(simulator.gui.animationSpeed*2); //required time for cleaning
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		dust = simulator.map.pickUp(currentPoint);
		if(dust!=null){
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was picked up.");
			simulator.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was picked up.");
			simulator.guiClear();
			addDustIntoBag(dust.volume); //add loaded dust items
		}
		else{
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") can not be picked up.");
		}
		return dust!=null;
//		return floorMap.pickUp(currentPoint);
	}
	
	/**
	 * Robot wipes at the current place for cleaning.
	 * @return returns true if the cleaning by wiping is succeeded, otherwise returns false.
	 */
	public boolean wipe() throws BatteryEmptyException, DustOverloadedException{
		Element dust;
			reduceBattery(_POWER_FOR_WIPE_); //energy consumption
		try {
			Thread.sleep(simulator.gui.animationSpeed*2); //required time for cleaning
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		dust = simulator.map.wipe(currentPoint);
		if(dust!=null){
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was wiped.");
			simulator.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") was wiped.");
			simulator.guiClear();
			addDustIntoBag(dust.volume); //add loaded dust items
		}
		else{
			System.out.println("\tDust at ("+currentPoint.x+","+currentPoint.y+") can not be wiped.");
		}
		return dust!=null;
//		return floorMap.wipe(currentPoint);
	}
	
	
	/**
	 * Charges the battery.
	 * @return returns true when the robot succeeded charging, otherwise returns false.
	 * (Charge is failed when robot is not in the same point of the station.)
	 */
	public boolean chargeBattery(){
		if(simulator.isLocatedOnStation()){
			//for visualization for charging (2011.6.3)
			try {
				Thread.sleep(_WAIT_FOR_CHARGING_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			setBattery(_MAX_BATTERY_);
//			this.battery = _MAX_BATTERY_;
			return true;
		}
		else return false;
	}

	/**
	 * Moves to next point
	 * (This action consumes power.)
	 * @param d@Direction that the robot should move
	 * @return if the movement succeeded, returns 0, otherwise,
	 * if the movement is failed, for example, by trying to move to out of the map, 
	 * returns  -1.
	 * @throws BatteryEmptyException 
	 */
	public int moveNext(Direction d) throws BatteryEmptyException{
		switch(d){
		case EAST:
			if(currentPoint.x+1 < simulator.map._WIDTH_){
				currentPoint.x++;
			}
			else return(-1);
			break;
		case WEST:
			if(0<currentPoint.x){
				currentPoint.x--;
			}
			else return(-1);
			break;
		case NORTH:
			if(currentPoint.y+1 < simulator.map._DEPTH_){
				currentPoint.y++;
			}
			else return(-1);
			break;
		case SOUTH:
			if(0<currentPoint.y){
				currentPoint.y--;
			}
			else return(-1);
			break;
		}
		simulator.moveRobot(currentPoint, d);
		simulator.println("\t Moved to ("+currentPoint.x + ","+currentPoint.y+").");
		System.out.println("\t Moved to ("+currentPoint.x + ","+currentPoint.y+").");
		reduceBattery(_POWER_FOR_MOVE_); //energy consumption
		return(0); //Normal termination
	}


	/**
	 * Robot finds the nearest object which type is specified as an argument.
	 * (This action consumes power.)
	 * @param elemType object type
	 * @return returns the point that the nearest specified object is located. 
	 * If the target object is not exist or the camera module of the robot is broken,
	 * returns null.
	 */
	public Point find(String elemType)throws BatteryEmptyException{
		if(!isCameraBroken){ //camera module is not broken
//			if(!elemType.equals("OtherRobot")){
				reduceBattery(_POWER_FOR_SEARCH_); //energy consumption
//			}

			Queue<Point> searchQueue = new LinkedList<Point>();
			//Store points that have already been stacked.
			HashSet<Point> checkedList = new HashSet<Point>();
//				System.out.println("\tCurrent point is "+"("+currentPoint.x+","+currentPoint.y+").");
			searchQueue.offer(currentPoint);
			checkedList.add(currentPoint);
			boolean isFound=false;
			while(searchQueue.size()!=0 && !isFound){
				Point target = searchQueue.poll();
//					System.out.println("\t\tCurrent target is "+"("+target.x+","+target.y+").");
				if(isLocated(target, elemType)){
//					dataStore.put(outPort, target);
					isFound = true;
					return(target);
//					mode = Mode.ACHIEVED;
				}
				else{
					Point north = new Point(target.x, target.y+1);
					Point south = new Point(target.x, target.y-1);
					Point east = new Point(target.x+1, target.y);
					Point west = new Point(target.x-1, target.y);
					if(isContainedInMap(north)){
						if(!isContainedInSet(checkedList, north)){
							searchQueue.offer(north);
							checkedList.add(north);
						}
					}
					if(isContainedInMap(south)){
						if(!isContainedInSet(checkedList, south)){
							searchQueue.offer(south);
							checkedList.add(south);
						}
					}
					if(isContainedInMap(east)){
						if(!isContainedInSet(checkedList, east)){
							searchQueue.offer(east);
							checkedList.add(east);
						}
					}
					if(isContainedInMap(west)){
						if(!isContainedInSet(checkedList, west)){
							searchQueue.offer(west);
							checkedList.add(west);
						}
					}
				}
			}
		}
		return(null);
	}
	
	/**
	 * Returns whether dust items remain in the field. (for checking termination condition)
	 * @return Returns true if there are no dust items in the field.
	 */
	public boolean isFinished()throws BatteryEmptyException{
		if(find("Dust")==null){
			System.out.println("\n*** All dust in this field is cleaned up. ***");
			return true;
		}
		else return false;
	}
	

	/**
	 * Unloads dust items into the dust bin
	 * @return returns true when the robot succeeded unloading, otherwise returns false.
	 * (Unload is failed when robot is not in the same point of the dust bin.)
	 */
	public boolean unloadDust(){
		if(simulator.isLocatedOnDustBin()){
			//for visualization for unloading (2011.6.3)
			try {
				Thread.sleep(_WAIT_FOR_UNLOADING_);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			setLoadAmount(0);
			simulator.println("\tDust was dumped into dustbin.");
			simulator.guiClear();
			return true;
		}
		else return false;
	}

	
	/**
	 * Gets the current battery level.
	 * @return current battery level (integer type).
	 */
	public int getBattery() {
		return battery;
	}


	/**
	 * Gets the current battery level. (with time interval for EXP2)
	 * @return battery level
	 */
	public void checkBatteryWithInterval() {
		// clear
		if(bt==null || !bt.isAlive()){
			isBatteryChecked = false;
			
			/** if the reply is returned by thread **/
			bt = new BatteryThread(this, serialID++);
			bt.start();
		}
		else{
//			System.out.println("bt "+bt.no+"is"+bt.getState());
		}
		
		/** if it directly replies **/
//		try {
//			Thread.sleep(_BATTERY_TIME_INTERVAL_);
//			visibleBattery = battery;
//			isBatteryChecked=true;
//		} catch (InterruptedException e) {
//			e.printStackTrace();
//		}

	}


	
////////////////////////////////////UTILITIES ////////////////////////////////////
	
	/**
	 * Checks whether the specified object exists at the specified point.
	 * @param p Point
	 * @param className Class name of the specified object
	 * @return returns true if the specified object is located at the specified point.
	 */
	private boolean isLocated(Point p, String className){
		return simulator.map.isLocated(p, className);
	}



	/**
	 * Changes battery level. (For Reducing the battery level from a button on GUI.)
	 * @param battery
	 */
	protected void setBattery(int battery) {
		if(battery < 0){
			this.battery = 0;
		}
		else{
			this.battery = battery;
		}
		System.out.println("\t\tRest battery: "+this.battery);
		simulator.batteryLabelChange(this.battery);
	}

	/**
	 * Changes the load amount. (For Adding the load amount from a button on GUI.)
	 * @param loadAmount
	 */
	protected void setLoadAmount(int loadAmount) {
		if(loadAmount <= _MAX_LOAD_AMOUNT_){
			this.loadAmount = loadAmount;
		}
		else{
			this.loadAmount = loadAmount;
//			this.loadAmount = _MAX_LOAD_AMOUNT_;
			System.err.println("VirtualRobot.java: OverloadItemsError");
		}
		System.out.println("\t\tload amount: "+this.loadAmount);
		simulator.loadAmountChange(this.loadAmount);
	}

	/**
	 * Gets the current load amount of the robot.
	 * @return current load amount (integer type).
	 */
	public int getLoadAmount() {
		return loadAmount;
	}


	/**
	 * Sets the robot's initial location (used by Simulator class)
	 * @param battery
	 */
	protected void setPoint() {
				this.currentPoint = simulator.map.initRobotPoint;
	}

	/**
	 * Decrease the battery level.
	 * @throws BatteryEmptyException 
	 */
	private void reduceBattery(int v) throws BatteryEmptyException{
		setBattery(this.battery - v);
//		this.battery = this.battery - v;
		if(isEmpty()){ //out of charge
			throw new BatteryEmptyException();
		}
	}

	@SuppressWarnings("serial")
	public class BatteryEmptyException extends Exception{
		public BatteryEmptyException(){}
	}
	
	/**
	 * Adds load amount.
	 */
	private void addDustIntoBag(int vol)throws DustOverloadedException{
		setLoadAmount(getLoadAmount()+vol); 
		if(_LIMIT_OF_GARBAGE_BAG_ > 0){
			if(this.loadAmount >= _LIMIT_OF_GARBAGE_BAG_){
				throw new DustOverloadedException();
			}
		}
	}

	@SuppressWarnings("serial")
	public class DustOverloadedException extends Exception{
		public DustOverloadedException(){}
	}
	

	
	/**
	 * Checks whether the specified point is included in the set of points.
	 * @param s Set of Points
	 * @param p Point
	 * @return if p in s returns true.
	 */
	private boolean isContainedInSet(Set<Point> s, Point p){
		for(Iterator<Point> i = s.iterator();i.hasNext();){
			Point comp_p= (Point)i.next();
			if(comp_p.equals(p)) return true;
		}
		return false;
	}

	/**
	 * Checks whether the battery is empty or not.
	 * @return returns true when the battery level is 0 (or less), otherwise, returns false.
	 */
	private boolean isEmpty(){
		if(battery<=0){
			System.out.println("\n*** Battery is Empty... ***");
			return true;
		}
		else return false;
	}
	
	/**
	 * Thread for battery check (for Exp2)
	 * @author h-nakagawa
	 *
	 */
	private class BatteryThread extends Thread{
	    private VirtualRobot vr;
	    private int no;
	    
	    public BatteryThread(VirtualRobot vr, int name) {
	    	this.vr = vr;
	    	this.no = name;
		}

	    public void run() {
	    	try {
				Thread.sleep(_WAIT_FOR_CHECK_BATTERY_);
				vr.visibleBattery = vr.battery;
				vr.isBatteryChecked=true;
			} catch (InterruptedException e) {
				e.printStackTrace();
//				System.out.println(no+" is interrupted!");
			}
//			System.out.println("** running *****");
			
		}
	}

}
