package org.nak.cleaning.map;

import java.awt.Point;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.nak.cleaning.CleaningMethod;
import org.nak.cleaning.object.Appearance;
import org.nak.cleaning.object.ChargeStation;
import org.nak.cleaning.object.Dust;
import org.nak.cleaning.object.DustBin;
import org.nak.cleaning.object.Element;
import org.nak.cleaning.object.Obstacle;

/**
 * Field map class<br>
 * @author Hiroyuki NAKAGAWA
 * @version 1.0
 * 
 */
public class FieldMap {
	private static FieldMap instance;
	public static int _WIDTH_;
	public static int _DEPTH_;
	
	public static final int _EMPTY_ = 0;
	public static final int _LIGHT_WAIGHT_DUST_ = 1;
	public static final int _MIDDLE_WAIGHT_DUST_ = 2;
	public static final int _HEAVY_WAIGHT_DUST_ = 3;
	public static final int _CHARGE_STATION_ = 98;
	
	public static final int _CLEANER_ROBOT_ = 99;
	
	//Point where obstacle is located (2011.06.11) 
	public Obstacle obstacle;

//	private Object lock = new Object();
	
	public Point initRobotPoint; //Point that robot is initially located 
	public HashMap<Point,Element> map;
	
	/**
	 * The constructor.
	 */
	public FieldMap(){
		super();
		//Robot is located at (0, 0) if not specified.
		initRobotPoint = new Point(0,0);
	
		instance = this;
	}

	public static FieldMap getFieldMap(){
		if(instance==null) instance = new FieldMap();
		return instance;
	}

	
	/**
	 * Returns map data representing as (Concurrent)HashMap
	 * @return
	 */
//	public ConcurrentHashMap<Point, Element> getHashMap() {
	public HashMap<Point, Element> getHashMap() {
//		synchronized (lock) {
			return map;
//		}
	}


	/**
	 * Removes a dust item located at the specified point from the map.
	 * @return removed (cleaned) dust item object
	 */
	private Element removeDust(Point p, CleaningMethod method){
		if(existsDust(p)){
			Element elem = map.get(p);
			switch(method){
			case SWEEP:
				if(elem.isAbleToSweep){
					map.remove(p);
					return elem;
				}
				break;
			case PICK_UP:
				if(elem.isLiftable){
					map.remove(p);
					return elem;
				}
				break;
			case WIPE:
				if(elem.isAbleToWipe){
					map.remove(p);
					return elem;
				}
				break;
			default:
			}
		}

		return null;
	}


///////////////////////////////////////////////////////////
////////////////  API for the robot action  ///////////////
///////////////////////////////////////////////////////////
	
	/**
	 * Checks whether the specified object exists at the specified point.
	 * @param p Point
	 * @param className Class name of the specified object
	 * @return returns true if the specified object is located at the specified point.
	 */
	public boolean isLocated(Point p, String className){
		if(isContainedInMap(p)){
			if(className.equals("Obstacle")){
				if(obstacle!=null  
						&& p.x == obstacle.point.x 
						&& p.y == obstacle.point.y) return true;
			}
			else{
				Element elem;
				if((elem=map.get(p))!=null){
//					if(map[p.x][p.y]==object) return true;
					if(className.equals("Dust")){
						if(elem instanceof Dust) return true;
					}
					else{
						if(className.equals("ChargeStation")){
							if(elem instanceof ChargeStation) return true;
						}
						else{
							if(className.equals("DustBin")){
								if(elem instanceof DustBin) return true;
							}
						}
					}
				}
			}
		}
		return false;
	}

	/**
	 * Returns whether dust items remain in the map. (for checking termination condition)
	 * @return Returns true if there are no dust items in the map.
	 */
	public boolean isFinished(){
		Set<Point> map_key = map.keySet();
		for(Iterator<Point> i= map_key.iterator(); i.hasNext();){
			Point point = (Point)i.next();
			if(isLocated(point, "Dust")) return false;
		}
		return true;
	}


	/**
	 * Checks whether the specified point is in the map.
	 * @param p Point
	 * @return returns if Point p is in the map, otherwise returns false.
	 */
	public boolean isContainedInMap(Point p){
		if(0<=p.x && p.x < _WIDTH_ && 0<=p.y && p.y < _DEPTH_)
			return true;
		else return false;
	}

	/**
	 * Checks whether a dust item is located at the specified point.
	 * @param p Point
	 * @return returns true if there is a dust item at the specified point.
	 */
	public boolean existsDust(Point p){
		if(isContainedInMap(p)){
			return isLocated(p,"Dust");
		}
		return false;
	}

	/**
	 * Gets appearance of the object located at the point p.
	 * @return Appearance of the object at the point p.
	 */
	public Appearance getAppearance(Point p){
		if(isContainedInMap(p)){
			Element elem = map.get(p);
			if(elem==null) return null;
			else return (elem.appearance);
		}
		return null;
	}


	/**
	 * Returns the result of sweeping at the point p.
	 * @return returns true if the cleaning by sweeping is succeeded, otherwise returns false.
	 */
	public Element sweep(Point p){
		return removeDust(p, CleaningMethod.SWEEP);
	}

	/**
	 * Returns the result of picking up at the point p.
	 * @return returns true if the cleaning by picking up is succeeded, otherwise returns false.
	 */
	public Element pickUp(Point p){
		return removeDust(p, CleaningMethod.PICK_UP);
	}
	
	/**
	 * Returns the result of wiping at the point p.
	 * @return returns true if the cleaning by wiping is succeeded, otherwise returns false.
	 */
	public Element wipe(Point p){
		return removeDust(p, CleaningMethod.WIPE);
	}

	

}
