package robot.de;

import jade.core.behaviours.ThreadedBehaviourFactory;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;

import org.dyncle.CLLoader;
import org.dyncle.ComponentBehaviour;
import org.dyncle.SASystem;
import org.nak.cleaning.Simulator;

import robot.de.comp.DisposeDust;
import robot.de.comp.PickUpDust;
import robot.de.comp.SweepDust;
import robot.de.comp.additional.MaintainBattery;
import robot.de.comp.additional.MaintainLoadAmount;


/**
 * CleaningRobot 
 * @version 5.0: Dynamic evolution version
 * @author h-nakagawa
 *
 */
@SuppressWarnings("serial")
public class CleaningRobot extends SASystem {
	
	
	//TODO: set components using XML file in the future
	DisposeDust disposeDust;
	MaintainBattery maintainBattery;
//	ComponentBehaviour approachObject;
	MaintainLoadAmount maintainLoadAmount;

	String path = "robot.de";

	
	public Simulator simulator;

	//Class for terminating threaded behaviors
	private ThreadedBehaviourFactory tbf = new ThreadedBehaviourFactory();

	
	/////////////////// Variables for Planning ///////////////////
	public RobotMode robotMode;

	//Whether planning is needed 
	//(it becomes true when all plans are executed or 
	//the robot detects errors that the robot can't deal with using current plan set.)
	public boolean isPlanningNeeded;
	
	//For the measurement of planning results
	public PlanningResult planningResult;

	public Queue<RobotEvent> eventQueue;
	

	
	/**
	 * Constructor
	 */
	public CleaningRobot() {
		super();
		
		simulator = Simulator.getSimulator();
		
		planningResult = new PlanningResult();
	}

	/**
	 * Sets up a cleaning robot
	 */
	protected void setup() {
		try {
			super.setup();
			System.out.println("*** Cleaning Robot ***");

			eventQueue = new LinkedList<RobotEvent>();
			robotMode = RobotMode.DISPOSE_DUST;

			
			isPlanningNeeded = true;
			
			ComponentBehaviour approachObject;

			// Register CL for Approaching Objects

			//specify class path (if the path is not found, it will be ignored)
			CLLoader ucLoader = new CLLoader("../Jade-SelfAdaptiveDemo/bin"); 
			approachObject = ucLoader.loadCL("robot.de.comp.ApproachObject", this);
			//			System.out.println(ucLoader.);
			classLoaderRepository.put("ApproachObject", ucLoader);

			standBy(approachObject);

			//add information of CL for approaching object
			//TODO: generalization and load from file
			addCLInfo(approachObject.name, path+"."+approachObject.name, 
					approachObject, 7, new ArrayList<String>());
			approachObject = null; //For preparing unload

			
			disposeDust = new DisposeDust(this);
			standBy(disposeDust);


			registerComponent("Cleaning", new SweepDust(this));
			registerComponent("Cleaning", new PickUpDust(this));
//			registerComponent("ApproachObject", approachObject);

			
			//add information of CL for cleaning
			ArrayList<String> al1 = new ArrayList<String>();
			al1.add("ApproachObject");
			addCLInfo(disposeDust.name, path+"."+disposeDust.name, 
					disposeDust, 5, al1);

			
			// CL for battery maintenance
			//** If it is added at run-time, following code should be comment out. **
			maintainBattery = new MaintainBattery(this);
			standBy(maintainBattery);
			ArrayList<String> al2 = new ArrayList<String>();
			al2.add("ApproachObject");
			addCLInfo(maintainBattery.name, path+"."+maintainBattery.name,
					maintainBattery, 1, al2);

			// CL for load amount management
			//** If it is added at run-time, following code should be comment out. **
//			maintainLoadAmount = new MaintainLoadAmount(this);
//			standBy(maintainLoadAmount);
//			ArrayList<String> al3 = new ArrayList<String>();
//			al3.add("ApproachObject");
//			addCLInfo(maintainLoadAmount.name, path+"."+maintainLoadAmount.name,
//					maintainLoadAmount, 3, al3);

			
			disposeDust.activate();
			
			while(!simulator.isReady()){
				Thread.sleep(1000);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Tasks for terminating the robot.
	 * (This method Overrides Agent.doDelete() method.)
	 */
	@Override
	public void doDelete(){
		System.out.println("Agent is kelled.");
		System.out.println("Size of Threaded Behaviour Factory: "+tbf.size());
		if(tbf.size()!=0){
			tbf.interrupt();
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		System.out.println("Size of Threaded Behaviour Factory: "+tbf.size());
		super.doDelete();
	}


}
