package robot.de;

import java.text.DecimalFormat;
import java.util.Iterator;

import jess.Defrule;
import jess.Fact;
import jess.JessException;
import jess.Rete;

/**
 * Class for storing the planning results
 * (This class does not have any impacts on the planning.)
 * @author h-nakagawa
 *
 */
public class PlanningResult {
	//Variables for measurement
	public int num_of_planning = 0; //number of planning execution times
	public int num_of_firedRules = 0; //number of fired rules
	public double total_planning_time = 0; //total elapsed time for Planning
	public int total_createdPlans = 0; //total number of generated plans
	public int total_assignedComps = 0; //total number of assigned components
	public int total_facts = 0; //total number of facts
	public int total_rules = 0; //total number of defined rules

	
	/**
	 * Accumulates results
	 * @param factsList
	 * @throws JessException 
	 */
	public void accumulateResult(Rete engine) throws JessException{
//	public void accumulateResult(Iterator<Fact> factsList){
		
		//extract fired rules (values of FIRED-RULES)
		int firedRulesNo = engine.fetch("FIRED-RULES").intValue(engine.getGlobalContext());
		System.out.println("#Fired rules: "+firedRulesNo);
		num_of_firedRules = num_of_firedRules+ firedRulesNo;
		

		//extract fact list
		// Note that it includes generated ones during planning
		int factNum=0;
		int ruleNum=0;
		int planNum=0;
		int compNum=0;
		Iterator<Fact> factsList = (Iterator<Fact>) engine.listFacts();
		for(Iterator<Fact> i = factsList;i.hasNext();){
			factNum++;
			Fact fact = (Fact)i.next();
//			System.out.println("fact-"+factNum+": "+fact.toString());
//			System.out.println("\tfact name: "+fact.getName());
			if(fact.getName().equals("MAIN::plan")){
				planNum++;
			}
			if(fact.getName().equals("MAIN::assignment")){
				compNum++;
			}
		}
		
		Iterator<Defrule> defrulesList = (Iterator<Defrule>) engine.listDefrules();
		for(Iterator<Defrule> i = defrulesList;i.hasNext();){
			ruleNum++;
			Defrule rule = (Defrule)i.next();
//			System.out.println("rule-"+ruleNum+": "+rule.toString());
			ruleNum++;
		}

//		System.out.println("\tsize of facts: "+factNum);
//		System.out.println("\tsize of plans: "+planNum);
//		System.out.println("\tsize of assigned components: "+compNum);
		total_createdPlans = total_createdPlans+planNum;
		total_assignedComps = total_assignedComps + compNum;
		total_facts = total_facts + factNum;
		total_rules = total_rules + ruleNum;

		//increment planning execution time
		num_of_planning++;

	}
	
	/**
	 * Accumulates elapsed time for planning
	 * @param time
	 */
	public void addElapsedTime(long time){
		total_planning_time = total_planning_time + time;
	}
	
	/**
	 * output the result
	 */
	public void printResults(){
		// Decimal format class
		DecimalFormat df = new DecimalFormat();
		// display format (decimal: double-digit)	
		df.applyLocalizedPattern("###.##");

		System.out.println("*** All dust is disposed.");
		System.out.println("\t#Planning: "+num_of_planning);
		System.out.println("\t#Elapsed time for planning: "+total_planning_time+" [ms]"+"\t(Avg: "+df.format(((double)total_planning_time/num_of_planning))+" [ms])");
		System.out.println("\t#Fired rules: "+num_of_firedRules+"\t\t(Avg: "+df.format(((double)num_of_firedRules/num_of_planning))+")");
		System.out.println("\t#Total generated plans: "+total_createdPlans+"\t(Avg: "+df.format(((double)total_createdPlans/num_of_planning))+")");
		System.out.println("\t#Total assigned components: "+total_assignedComps+"\t(Avg: "+df.format(((double)total_assignedComps/num_of_planning))+")");
		System.out.println("\t#Total facts: "+total_facts+"\t\t(Avg: "+df.format(((double)total_facts/num_of_planning))+")");
		System.out.println("\t#Total rules: "+total_rules+"\t\t(Avg: "+df.format(((double)total_rules/num_of_planning))+")");

	}
}
