package robot.de.comp;

import java.awt.Point;

import org.dyncle.ADTypeComponent;
import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;



/**
 *  Component for approaching target objects <br/>
 * @author Hiroyuki NAKAGAWA
 */
public class ApproachObject extends ADTypeComponent {
	String targetType;
	Port<Object> providedPort;
	boolean isFinished;
	MoveTo moveTo;
	Find find;
	ComponentBehaviour getPosition;

	Port<Point> gpsPort;
	Port<Object> findPort;
	Port<Point> moveToPort;
	
	Point targetPoint; //destination
	Point currentPoint;

	public boolean done(){
		return isFinished;
	}

	public ApproachObject(SASystem agent) {
		super(agent, "ApproachObject", "ApproachObject");
//		name = "ApproachObject";
		gpsPort = new Port<Point>("CurrentPoint", PortType.REQUIRED,this);
		addPort(gpsPort);
		findPort = new Port<Object>("TargetPoint", PortType.REQUIRED,this);
		addPort(findPort);
		moveToPort = new Port<Point>("DestinationPoint", PortType.REQUIRED,this); 
		addPort(moveToPort);
		providedPort = new Port<Object>("TargetPoint", PortType.PROVIDED,this);
		addPort(providedPort);

		getPosition = new GetPosition(agent);
		agent.standBy(getPosition);
		connectPorts(gpsPort, getPosition.getProvidedPorts().get(0));

		getPosition.activate();
		moveTo = new MoveTo(agent);
		agent.standBy(moveTo);
		find = new Find(agent);
		agent.standBy(find);
		
		targetPoint = null;

	}
	
	/**
	 * Execution task
	 */
	public void perform() {
		//get current position
		try {
			if(gpsPort.isArrive()){
				currentPoint = (Point)gpsPort.get();
//				System.out.println("Acquired current point: "+currentPoint);
			}
		} catch (PortException e) {
			e.printStackTrace();
		}

		//get target object's type from the other control loops
		if(targetType==null){
			try {
				if(providedPort.isArrive()){
					targetType = (String)providedPort.get();
				}
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		else{
			if(find.mode == Mode.WAITING){
				System.out.println("Component: ApproachObject");
				connectPorts(findPort, find.getProvidedPorts().get(0));
				try {
					findPort.put((Object)targetType);
				} catch (PortException e) {
					e.printStackTrace();
				}
				find.activate();
			}
			if(find.mode==Mode.ACHIEVED){
				if(moveTo.mode == Mode.WAITING){
					
					targetPoint = null;
					if(findPort.isArrive()){
						try {
							targetPoint = (Point)findPort.get();
							if(targetPoint!=null){
								System.out.println("\t"+targetType+" is found at ("+targetPoint.x+","+targetPoint.y+").");
							}
						} catch (PortException e) {
							e.printStackTrace();
						}
					}
					connectPorts(moveToPort, moveTo.getProvidedPorts().get(0));
					try {
						moveToPort.put(targetPoint);
					} catch (PortException e) {
						e.printStackTrace();
					}
					moveTo.activate();
				}
			}
			if(moveTo != null && moveTo.mode == Mode.ACHIEVED){
				if(currentPoint.x==targetPoint.x && currentPoint.y==targetPoint.y){
					mode = Mode.ACHIEVED;
					System.out.println("Arrive at ("+currentPoint.x + ","+currentPoint.y+").");
				}
				else mode= Mode.NOT_ACHIEVED;
				find.passivate();
				moveTo.passivate();
			}
		}
	}
	
	/** Passivates the component's service */
	public void passivate(){
		super.passivate();
		targetType = null;
		targetPoint = null;
		find.passivate();
		moveTo.passivate();
	}
}