package robot.de.comp;

import java.awt.Point;

import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;

import robot.de.CleaningRobot;

/**
 * Component for finding target object by using camera <br/>
 * @author Hiroyuki NAKAGAWA
 */
@SuppressWarnings("serial")
public class DiscoverObjectUsingCamera extends ComponentBehaviour {
	String targetType;
	Port<Object> outputPort; //output data: the nearest point that the specified object exists


	/**
	 * Constructor
	 * @param eType specified object type
	 */
	 public DiscoverObjectUsingCamera(CleaningRobot agent) {
		super(agent);
		outputPort = new Port<Object>("TargetPoint", PortType.PROVIDED,this);
		this.addPort(outputPort);
	}

		/**
		 * Execution task
		 */
	public void perform() {
		if(targetType==null){
			try {
				targetType = (String)outputPort.get();
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		else{
//			System.out.println("Component: DiscoverObjectUsingCamera");
			Point targetPoint = ((CleaningRobot)myAgent).simulator.find(this.targetType);
			if(targetPoint!=null){//target object is found
				try {
					outputPort.put(targetPoint);
				} catch (PortException e) {
					e.printStackTrace();
				}
				mode = Mode.ACHIEVED;
			}
			else{//target object is not found (camera may be broken)
				try {
					outputPort.put(null);
				} catch (PortException e) {
					e.printStackTrace();
				}
				if(((CleaningRobot)myAgent).simulator.isCameraBroken()){
					mode = Mode.NOT_ACHIEVED;
				}
				else{//judge that the target objects do not exist in the field
					mode = Mode.ACHIEVED;
				}
			}
		}
	}
	
	/** Passivates the component's service */
	public void passivate(){
//		System.out.println("\tpassivate DiscoverObjectUsingCamera");
		super.passivate();
		targetType = null;
	}

}
