package robot.de.comp;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import jess.Fact;
import jess.JessException;
import jess.RU;
import jess.Value;

import org.dyncle.ADTypeComponent;
import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Plan;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.nak.cleaning.object.Appearance;

import robot.de.CleaningRobot;
import robot.de.RobotEvent;

/**
 * Component for managing cleaning <br/>
 * @author Hiroyuki NAKAGAWA
 */
public class DisposeDust extends ADTypeComponent {
	boolean isFirstTime=true;
	// when this component is activate,  it prints out the component name just once

	public Appearance targetAppearance;
	private boolean isFirstCleaning;

	ObserveDustAppearanceAndResult observe;
//	private List<String> failedComponents; //Cleaning methods that already failed
	private List<Fact> eventList; //List of event that occurred during cleaning

	Point nextDust;
	Port<Object> approachPort;
	ComponentBehaviour approachObject;

	ComponentBehaviour currentComponent; //current running component

	//----------Variables for planner (2011.01.11)------------
	String jessFile= "./resources/divided/disposeRules.clp";
	private Queue<RobotEvent> eventQueueForCleaning;
	Plan currentPlan;

	//----------Variable for Planner (JSHOP2) (2011.02.19)------------
	String scriptFile= "sh/jshop/jshop2cleaning.sh";



	public DisposeDust(CleaningRobot agent) {
		super(agent);
		name = "DisposeDust";

		eventQueueForCleaning = new LinkedList<RobotEvent>();

		observePort = new Port<Appearance>("ObserveResult",PortType.REQUIRED,this);
//		this.<Appearance>setObservePort();
		observe = new ObserveDustAppearanceAndResult((CleaningRobot)myAgent);
		connectPorts(observePort, observe.getProvidedPorts().get(0));
		myAgent.standBy(observe);

		approachPort = new Port<Object>("TargetPoint",PortType.REQUIRED,this); 
		//Send:"Dust": String, Receive: Point of the nearest dust item

		//trial program (dynamic addition)
//		approachDust = myAgent.findService("ApproachDust").get(0);
//		approachDust = new ApproachDust(agent);
//		standBy(approachDust);

	}

	/**
	 * Execution task
	 */
	public void perform() {
		if(isFirstTime){
//			System.out.println("Component:DisposeDust");
			isFirstTime = false;
			
			//------use Planning (2011.06.01)-----------------
//			planList.reset();
//			makePlan(scriptFile);
//			planList.printAllPlans();
//			currentPlan = planList.getNextPlan();
//			System.out.println("First Plan");
//			currentPlan.printPlan();
//			currentComponent = myAgent.findComponent(currentPlan.getPlanName());
//			myAgent.standBy(currentComponent);
			//------use Planning (2011.06.01)-----------------
			approachObject = myAgent.getCL("ApproachObject");
//			approachObject = myAgent.findComponent("ApproachObject");
			currentComponent = approachObject;
//			myAgent.standBy(currentComponent);
		}
		if(currentComponent==null){ //ApproachObject service is not provided
			isFirstTime = true;
		}
		else{
			if(currentComponent.getName().equals("ApproachObject")){
				if(!approachPort.isConnected()){// try to connect
//					System.out.println(this.getName()+": try to connect...");
////					connectPorts(approachPort, currentComponent.getProvidedPorts().get(0));
					if((approachObject = myAgent.getCL("ApproachObject"))!=null){
						connectPorts(approachPort, approachObject.getProvidedPorts().get(0));
					}
					else{//ApproachObject service is not provided
						isFirstTime = true;
					}
				}
				else{
//					System.out.println("DisposeDust: connected");
					if(currentComponent.mode == Mode.WAITING){
						isFirstCleaning=true;
						try {
							approachPort.put((Object)"Dust");
						} catch (PortException e) {
							e.printStackTrace();
						}
						currentComponent.activate();
					}

					if(currentComponent.mode==Mode.ACHIEVED){
						currentComponent.passivate(); //ApproachObject
//						currentComponent = myAgent.findComponent(currentPlan.getPlanName());
//						currentComponent = myAgent.findComponent("Cleaning");
						System.out.println("Change to cleaning method.");
						currentComponent = myAgent.findComponent("SweepDust");
						myAgent.standBy(currentComponent);
					}
					if(currentComponent.mode==Mode.NOT_ACHIEVED){
						mode = Mode.NOT_ACHIEVED;
					}
//					System.out.println("current state "+currentComponent.mode);
				}
			}
	//------ use Planning (2011.06.01)(replace above ApproachObject block)-----------------
//			if(currentComponent.getName().equals("Find")){
//				if(currentComponent.mode == Mode.WAITING){
//					isFirstCleaning=true;
//					connectPorts(findPort, currentComponent.getProvidedPorts().get(0));
//					try {
//						findPort.put((Object)"Dust");
//					} catch (PortException e) {
//						e.printStackTrace();
//					}
//					currentComponent.activate();
//				}
//				if(currentComponent.mode==Mode.ACHIEVED){
//					nextDust = null;
//					if(findPort.isArrive()){
//						try {
//							nextDust = (Point)findPort.get();
//							if(nextDust !=null){
//								System.out.println("\tDust is found at ("+nextDust.x+","+nextDust.y+").");
//							}
//						} catch (PortException e) {
//							e.printStackTrace();
//						}
//					}
//					currentComponent.passivate(); //find
//					currentPlan = planList.getNextPlan();//moveTo
//					System.out.println("NextPlan: "+currentPlan.getPlanName());
//					currentComponent = myAgent.findComponent(currentPlan.getPlanName());
//					myAgent.standBy(currentComponent);
//				}
//			}
//			if(currentComponent.getName().equals("MoveTo")){
//				if(currentComponent.mode == Mode.WAITING){
//					connectPorts(moveToPort, currentComponent.getProvidedPorts().get(0));
//					try {
//						moveToPort.put(nextDust);
//					} catch (PortException e) {
//						e.printStackTrace();
//					}
//					currentComponent.activate();
//				}
//				if(currentComponent.mode==Mode.ACHIEVED){
//					currentComponent.passivate(); //moveTo
//					currentPlan = planList.getNextPlan();//cleaning method (Default: sweepDust)
//					System.out.println("NextPlan: "+currentPlan.getPlanName());
//					currentComponent = myAgent.findComponent(currentPlan.getPlanName());
//					myAgent.standBy(currentComponent);
//				}
//				if(currentComponent.mode==Mode.NOT_ACHIEVED){
//					mode = Mode.NOT_ACHIEVED;
//				}
//			}
			//------use Planning (2011.06.01)-----------------
			
			if(currentComponent.getServiceType().equals("Cleaning")){
				if(currentComponent.mode == Mode.WAITING){
					//start cleaning
						if(observe.mode == Mode.WAITING && isFirstCleaning){
//							failedComponents = new ArrayList<String>();
							targetAppearance = null;
							eventList = new ArrayList<Fact>();

							isFirstCleaning=false;
							if(observePort.isArrive()){//obtain appearance of dust item
								try {
									observePort.get();//dump previous observed results
								} catch (PortException e) {
									e.printStackTrace();
								} 
							}

							observe.activate();
						}
						if(observePort.isArrive()){//obtain appearance of dust item
							try {
								targetAppearance = (Appearance)observePort.get();
								System.out.println("Got appearance: "+targetAppearance);
								observe.passivate();
							} catch (PortException e) {
								e.printStackTrace();
							}
							if(targetAppearance!=Appearance.NOTHING){
								//dust item remains (success to obtain appearance or cleaning failure)
								System.out.println("\tIdentified object type :"+targetAppearance.toString());
									
								try {
								   long start = System.currentTimeMillis();
									//prepare jess execution
									setRules(jessFile);
									//construct facts and register them
									setEnvironmentFacts();
									

									//------use Planning (2011.06.01)-----------------
									//run jess
									engine.executeCommand("(run-once)");
									//------use Planning(2011.06.01)-----------------
									
									//extract results (selected cleaning method)
									Value cleaningMethodValue = engine.fetch("CLEANING-METHOD");
									if(cleaningMethodValue==null){
										System.err.println("DisposeSingleDust: Not defined Error");
										currentComponent = myAgent.findService("Cleaning").get(0);
			
									}
									else{
										String cleaningMethod = cleaningMethodValue.stringValue(engine.getGlobalContext());
										System.out.println("Determined cleaning method: "+cleaningMethod);
										currentComponent.terminate();
										currentComponent = myAgent.findComponent(cleaningMethod);
//										num_of_planning++;
									}

									//accumulate planing result
									((CleaningRobot)myAgent).planningResult.accumulateResult(engine);

									//planning finished
									//Calculate and accumulate required time
//								   long stop = System.currentTimeMillis();
//								   System.out.println("\tElapsed time: "+(stop-start)+"ms");
//								   ((CleaningRobot)myAgent).planningResult.addElapsedTime(stop - start);


									if(currentComponent==null){//There are no alternative cleaning methods
										mode = Mode.NOT_ACHIEVED;
										System.err.println("All cleaning method couldn't make it.");
									}
									else{
										myAgent.standBy(currentComponent);
										currentComponent.activate();
									}
									
								} catch (JessException e) {
									e.printStackTrace();
								}
							}
						}
				}
				if(currentComponent.mode == Mode.ACHIEVED){
					if(!observePort.isArrive()){ //check whether dust item still remains
//						System.out.println("observe after cleaning.");
						observe.activate();
					}
					else{//obtain appearance of dust item
						try {
							targetAppearance = (Appearance)observePort.get();
							System.out.println("Got appearance: "+targetAppearance);
							observe.passivate();
						} catch (PortException e) {
							e.printStackTrace();
						}
						if(targetAppearance==Appearance.NOTHING){//cleaning succeeded
							System.out.println("Dispose finished");
							currentComponent.passivate();
							currentComponent.terminate();
							currentComponent = null;
//							isCleaningFinished = true;
							eventQueueForCleaning.clear();
							
							//disconnect from ApproachObject
							disconnectPorts(approachPort, approachObject.getProvidedPorts().get(0));
							approachObject = null;

							isFirstTime = true; //TODO: more elegant description

							if(((CleaningRobot)myAgent).simulator.isFinished()){
								//all dust items are cleaned
								this.mode = Mode.ACHIEVED;
								((CleaningRobot)myAgent).isPlanningNeeded = true;
							}
							
						}
						else{//cleaning is failed
//							cleaningBehavior.finalize();
//							failedComponents.add(cleaningBehavior.name);
							
							//issue RobotEvent
							System.out.println("cleaning failed");
							RobotEvent rEvent = new RobotEvent("cleaning-failed", currentComponent.getName(), "Dust", ((CleaningRobot)myAgent).simulator.getCurrentPoint());
							eventQueueForCleaning.offer(rEvent);
							currentComponent.passivate(); //try another cleaning method
//							currentComponent.finalize();
							observe.activate();
						}
					}
				}
			}			
		}
		

	}

	/**
	 * Makes facts for jess execution (only about cleaning).
	 */
	private void setEnvironmentFacts(){
		try {
			//construct facts
			//robot's information
			Fact f = new Fact("robot", engine);
//			f.setSlotValue("location", new Value("0-0", RU.ATOM));
			f.setSlotValue("dust-num", new Value(0, RU.INTEGER));
			f.setSlotValue("mode", new Value("dispose-dust", RU.ATOM));
			engine.assertFact(f);

			//dust item's information ("appearance" is important)
			f = new Fact("object", engine);
			f.setSlotValue("type", new Value("dust", RU.ATOM));
//			f.setSlotValue("name", new Value("dust-A", RU.ATOM));
//			f.setSlotValue("location", new Value("X_dust-Y_dust", RU.ATOM));
			f.setSlotValue("appearance", new Value(targetAppearance.toString(), RU.ATOM));
			engine.assertFact(f);

			//make facts from the detected event
			while(eventQueueForCleaning.size()>0){
				RobotEvent rEvent = eventQueueForCleaning.poll();
				Fact eventfact = new Fact("event", engine);
				eventfact.setSlotValue("type", new Value(rEvent.getType(), RU.ATOM));
				eventfact.setSlotValue("action", new Value(rEvent.getAction(), RU.ATOM));
				eventfact.setSlotValue("location", new Value(rEvent.getLocation().toString(), RU.ATOM));
				if(rEvent.getType().equals("cleaning-failed")){
					eventfact.setSlotValue("object", new Value("dust", RU.ATOM));
				}
				eventList.add((Fact)eventfact.clone());
//				engine.assertFact(f);
			}
			
			//register detected event (fact) to jess
			for(Iterator<Fact> i = eventList.iterator();i.hasNext();){
				Fact fact = (Fact)i.next();
				engine.assertFact(fact);
			}

		} catch (JessException e) {
			e.printStackTrace();
		}
	}

	/** Activates the component's service */
	public void activate(){
		super.activate();
		isFirstTime = true;
		
		isFirstCleaning =true;
	}

	/** Passivates the component's service */
	public void passivate(){
		super.passivate();
//		System.out.println("\tpassivate DisposeDust");

		if(approachObject!=null){
			approachObject.passivate();
			approachObject = null;
		}

		if(currentComponent!=null){
			currentComponent.passivate();
			currentComponent = null;
		}

		nextDust = null;
		isFirstTime = true;
		
		//------use Planning (2011.06.01)-----------------
//		planList.reset();
		//------use Planning(2011.06.01)-----------------
		
		observe.passivate();
		if(observePort.isArrive()){//clean remained collected data
			try {
				targetAppearance = (Appearance)observePort.get();
				System.out.println("Remove left object (apprearance): "+targetAppearance);
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
	}
}

