package robot.de.comp;

import java.awt.Point;

import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;

import robot.de.CleaningRobot;

/**
 * Component for finding target objects <br/>
 * @author Hiroyuki NAKAGAWA
 */
public class Find extends ComponentBehaviour {
	String targetType;
	Point targetPoint;
	Port<Object> providedPort;
	
	DiscoverObjectUsingCamera discoverUsingCamera = null;
	Port<Object> findPort;

	ComponentBehaviour discoverByRandomMove=null;
	boolean isCameraActive;

	
	/**
	 * Constructor
	 */
	public Find(SASystem agent) {
		super(agent, "Find", "Find");
		findPort = new Port<Object>("TargetPoint",PortType.REQUIRED, this);
		
		providedPort = new Port<Object>("TargetPoint", PortType.PROVIDED, this);
		this.addPort(providedPort);
		
		//normally Use camera to find objects
		isCameraActive = true;
		discoverUsingCamera = new DiscoverObjectUsingCamera((CleaningRobot)myAgent);
		connectPorts(findPort, discoverUsingCamera.getProvidedPorts().get(0));
		myAgent.standBy(discoverUsingCamera);
	}
	
	/**
	 * Execution task
	 */
	public void perform() {
		if(targetType==null){
			try {
				targetType = (String)providedPort.get();
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		else{
			if(isCameraActive){ //using camera
				if(discoverUsingCamera.mode == Mode.WAITING){
					System.out.println("Component: Find");
					discoverUsingCamera.activate();
					try {
						System.out.println("\ttargetType: "+targetType);
						findPort.put((Object)targetType);
					} catch (PortException e) {
						e.printStackTrace();
					}
				}
				if(discoverUsingCamera.mode==Mode.ACHIEVED){//succeeded
					try {
						targetPoint = (Point)findPort.get();
						providedPort.put(targetPoint);
					} catch (PortException e) {
						e.printStackTrace();
					}
					this.mode = Mode.ACHIEVED;
					discoverUsingCamera.passivate();

				}
				if(discoverUsingCamera.mode == Mode.NOT_ACHIEVED){	//judge as camera failure
					isCameraActive=false;
					discoverUsingCamera.terminate();
					discoverUsingCamera = null;
					System.out.println("Sencing failed and change to rondomLyMoveToFindDust component.");
				}
			}
			else{ //camera is out of order
				if(discoverByRandomMove==null){
					discoverByRandomMove = new DiscoverObjectByRandomWalk((CleaningRobot)myAgent, targetType);
					myAgent.standBy(discoverByRandomMove);
				}
				else{
					if(discoverByRandomMove.mode == Mode.ACHIEVED){//succeeded
						try {
							targetPoint = (Point)findPort.get();
							providedPort.put(targetPoint);
						} catch (PortException e) {
							e.printStackTrace();
						}
						this.mode = Mode.ACHIEVED;
						discoverByRandomMove = null;
					}
					else{
						if(discoverByRandomMove.mode == Mode.NOT_ACHIEVED){
							mode = Mode.NOT_ACHIEVED;
						}
					}
				}
			}
		}
	}
	
	/** Passivates the component's service */
	public void passivate(){
//		System.out.println("\tpassivate Find");
		super.passivate();

		//TODO: generalization
		if(discoverUsingCamera!=null){
			discoverUsingCamera.passivate();
		}

		if(discoverByRandomMove!=null){
			discoverByRandomMove.terminate();
			discoverByRandomMove = null;
		}
		
		//port initialization
		while(providedPort.isArrive()){
			try {
				providedPort.get();
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		targetType = null;
	}
}
