package robot.de.comp.additional;

import java.awt.Point;

import org.dyncle.ADTypeComponent;
import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;
import org.nak.cleaning.VirtualRobot;

import robot.de.CleaningRobot;

/**
 * Component for maintaining battery level <br/>
 * @author Hiroyuki NAKAGAWA
 */
@SuppressWarnings("serial")
public class MaintainBattery extends ADTypeComponent {
	ChargeBattery chargeBattery = null;
	ObserveBatteryLevel observe;
	static final int _BATTERY_THRESHOLD_ = 30;
	int currentBattery = VirtualRobot._MAX_BATTERY_;
	ComponentBehaviour currentComponent;
	
	ComponentBehaviour approachObject;
	Point station;
	Port<Object> approachPort;

	private boolean canStart;
	
	public MaintainBattery(SASystem agent) {
		super(agent);
		name = "MaintainBattery";
		observePort = new Port<Integer>("ObserveResult",PortType.REQUIRED,this);
		observe = new ObserveBatteryLevel(myAgent);
		connectPorts(observePort, observe.getProvidedPorts().get(0));
		myAgent.standBy(observe);
		
		approachPort = new Port<Object>("TargetPoint",PortType.REQUIRED,this);
		//Send:"ChargeStation": String, Receive: Point of the nearest station

		observe.activate();
		canStart = false;
	}

	/**
	 * Execution task
	 */
	public void perform() {
		if(!canStart){
				canStart = true;
				approachObject = myAgent.getCL("ApproachObject");

				System.out.println("*** Robot noticed the battery level is low.");
	
				currentComponent = approachObject;
		}
		if(currentComponent==null){ //ApproachObject service is not provided
			canStart = false;
		}
		else{
			if(currentComponent.getName().equals("ApproachObject")){
				if(!approachPort.isConnected()){// try to connect
//					System.out.println(this.getName()+": try to connect...");
////					connectPorts(approachPort, currentComponent.getProvidedPorts().get(0));
					if((approachObject = myAgent.getCL("ApproachObject"))!=null){
						connectPorts(approachPort, approachObject.getProvidedPorts().get(0));
						currentComponent = approachObject; //2012.6.6
					}
				}
				else{
//					System.out.println(this.getName()+": connected");
					switch(currentComponent.mode){
					case WAITING:
//						isFirstCleaning=true;
						try {
							approachPort.put((Object)"ChargeStation");
						} catch (PortException e) {
							e.printStackTrace();
						}
						currentComponent.activate();
						break;

					case ACHIEVED:
						currentComponent.passivate(); //ApproachObject
//						currentComponent = myAgent.findComponent(currentPlan.getPlanName());
//						currentComponent = myAgent.findComponent("ChargeBattery");
						currentComponent = new ChargeBattery((CleaningRobot)myAgent);
						myAgent.standBy(currentComponent);
						currentComponent.activate();
						break;

					case NOT_ACHIEVED:
						mode = Mode.NOT_ACHIEVED;
					}
				}
			}
			if(currentComponent.getName().equals("ChargeBattery")){
				if(currentComponent.mode == Mode.WAITING){
					//start charging
					if(((CleaningRobot)myAgent).simulator.isLocatedOnStation()){
//						myAgent.standBy(currentComponent);
						currentComponent.activate();
//					if(fillBattery == null){
//						fillBattery = new ChargeBattery((CleaningRobot)myAgent);
//						myAgent.standBy(fillBattery);
//						fillBattery.activate();
					}
				}
				if(currentComponent.mode == Mode.ACHIEVED){
					try {
						if(observePort.isArrive()){
							currentBattery = ((Integer)observePort.get()).intValue();

							//wait to finish the charging
							if(currentBattery > _BATTERY_THRESHOLD_ ){
								System.out.println("Changing the behavior...");
								currentComponent.passivate();
								currentComponent.terminate();
								currentComponent = null;
								
								mode = Mode.WAITING;

								//disconnect from ApproachObject
								disconnectPorts(approachPort, approachObject.getProvidedPorts().get(0));
								approachObject = null;
								
								canStart = false;
							}
							else{
//								System.out.println("Waiting battery recharging...");
							}
						}
					} catch (PortException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	@Override
	public boolean activateCondition(){
		try {
			if(observePort.isArrive()){
				currentBattery = ((Integer)observePort.get()).intValue();
			}
		} catch (PortException e) {
			e.printStackTrace();
		}
		return ( currentBattery <= _BATTERY_THRESHOLD_);
	}
	
	/** Passivates the component's service */
	public void passivate(){
		if(approachObject!=null){
			approachObject.passivate();
			approachObject = null;
		}
//		if(currentComponent!=null) currentComponent.passivate();
		if(currentComponent instanceof ChargeBattery){
			currentComponent.terminate();
		}
		currentComponent = null;
		
		canStart = false;
		super.passivate();
	}
}