package robot.de.comp.additional;

import java.awt.Point;

import org.dyncle.ADTypeComponent;
import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;

import robot.de.CleaningRobot;

/**
 * Component for managing load amount <br/>
 * @author Hiroyuki NAKAGAWA
 */
@SuppressWarnings("serial")
public class MaintainLoadAmount extends ADTypeComponent {
	UnloadDust unloadDust = null;
	ObserveLoadAmount observe;
	
	static final int _LOAD_AMOUNT_THRESHOLD_ = 100;
	public int currentLoadAmount = 0;
	ComponentBehaviour currentComponent;

	ComponentBehaviour approachObject;
	Point dustBin;
	Port<Object> approachPort;

	private boolean canStart;
	
	public MaintainLoadAmount(SASystem agent) {
		super(agent);
		name = "MaintainLoadAmount";
		observePort = new Port<Integer>("ObserveResult",PortType.REQUIRED,this);
		observe = new ObserveLoadAmount(myAgent);
		connectPorts(observePort, observe.getProvidedPorts().get(0));
		myAgent.standBy(observe);
		
		approachPort = new Port<Object>("TargetPoint",PortType.REQUIRED,this); 
		//Send:"DustBin": String, Receive: Point of the nearest dust bin

		observe.activate();
		canStart = false;
	}

	/**
	 * Execution task
	 */
	public void perform() {
		if(!canStart){
			canStart = true;
			approachObject = myAgent.getCL("ApproachObject");


			currentComponent = approachObject;
		}
		if(currentComponent==null){ //ApproachObject service is not provided
			canStart = false;
		}
		else{
			if(currentComponent.getName().equals("ApproachObject")){
				if(!approachPort.isConnected()){// try to connect
//					System.out.println(this.getName()+": try to connect...");
////					connectPorts(approachPort, currentComponent.getProvidedPorts().get(0));
					if((approachObject = myAgent.getCL("ApproachObject"))!=null){
//						System.out.println("ApproachObject is found."); //2012.6.6
						connectPorts(approachPort, approachObject.getProvidedPorts().get(0));
						currentComponent = approachObject; //2012.6.6
					}
				}
				else{
//					System.out.println(this.getName()+": connected");
					switch(currentComponent.mode){
					case WAITING:
//						isFirstCleaning=true;
						try {
							approachPort.put((Object)"DustBin");
						} catch (PortException e) {
							e.printStackTrace();
						}
						currentComponent.activate();
						break;

					case ACHIEVED:
						currentComponent.passivate(); //ApproachObject
						currentComponent = new UnloadDust((CleaningRobot)myAgent);
						myAgent.standBy(currentComponent);
						currentComponent.activate();
						break;

					case NOT_ACHIEVED:
						mode = Mode.NOT_ACHIEVED;
					}
				}
			}
			
			if(currentComponent.getName().equals("UnloadDust")){
				if(currentComponent.mode == Mode.WAITING){
					//start unloading dust items
					if(((CleaningRobot)myAgent).simulator.isLocatedOnDustBin()){
						currentComponent.activate();
					}
				}
				if(currentComponent.mode == Mode.ACHIEVED){
					try {
						if(observePort.isArrive()){
							currentLoadAmount = ((Integer)observePort.get()).intValue();
							if(currentLoadAmount < _LOAD_AMOUNT_THRESHOLD_){
								//Unloading dust items is completed
								currentComponent.passivate();
								currentComponent.terminate();
								currentComponent = null;
								
								mode = Mode.WAITING;

								//disconnect from ApproachObject
								disconnectPorts(approachPort, approachObject.getProvidedPorts().get(0));
								approachObject = null;
								
								canStart = false;
							}
							else{
//								System.out.println("Waiting load amount measured...");
							}
						}
					} catch (PortException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	@Override
	public boolean activateCondition(){
		try {
			if(observePort.isArrive()){
				currentLoadAmount = ((Integer)observePort.get()).intValue();
			}
		} catch (PortException e) {
			e.printStackTrace();
		}
		return ( currentLoadAmount	>= _LOAD_AMOUNT_THRESHOLD_);
	}
	
	/** Passivates the component's service */
	public void passivate(){
		if(approachObject!=null){
			approachObject.passivate();
			approachObject = null;
		}
//		if(currentComponent!=null) currentComponent.passivate();
		if(currentComponent instanceof UnloadDust){
			currentComponent.terminate();
		}
		currentComponent = null;
		canStart = false;
		super.passivate();
	}
}