package robot.de.comp.alternative;


import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import org.dyncle.ADTypeComponent;
import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;
import org.nak.cleaning.Direction;

import robot.de.comp.Find;
import robot.de.comp.GetPosition;


/**
 * (Updated) Component for approaching target objects with obstacle avoidance <br/>
 * @author Hiroyuki NAKAGAWA
 */
public class ApproachObject extends ADTypeComponent {
	String targetType;
	Port<Object> providedPort;
	boolean isFinished;
	ComponentBehaviour moveTo;
	ComponentBehaviour find;
	ComponentBehaviour getPosition;

	Port<Point> gpsPort;
	Port<Object> findPort;
//	Port<TargetAndAvoidPoints> moveToPort;
	Port<Point> moveToPort;
	
	Point targetPoint; //destination
	Point currentPoint;
	
	
	Point obstaclePoint; //obstacle located place (point)
	List<Point> avoidPoints; //list of obstacle located points
	List<Direction> possibleDirection; //possible directions to move forward
	Direction previousDirection; //for preventing backward

	ComponentBehaviour currentComponent;

	public boolean done(){
		return isFinished;
	}

	public ApproachObject(SASystem agent) {
		super(agent, "ApproachObject", "ApproachObject");
		gpsPort = new Port<Point>("CurrentPoint", PortType.REQUIRED,this);
		addPort(gpsPort);
		findPort = new Port<Object>("TargetPoint", PortType.REQUIRED,this);
		addPort(findPort);
//		moveToPort = new Port<TargetAndAvoidPoints>("DestinationPoint", PortType.REQUIRED,this); 
		moveToPort = new Port<Point>("DestinationPoint", PortType.REQUIRED,this); 
		addPort(moveToPort);
		providedPort = new Port<Object>("TargetPoint", PortType.PROVIDED,this);
		addPort(providedPort);

		getPosition = new GetPosition(agent);
		agent.standBy(getPosition);
		connectPorts(gpsPort, getPosition.getProvidedPorts().get(0));

		getPosition.activate();
		moveTo = new MoveTo(agent);
		agent.standBy(moveTo);
		find = new Find(agent);
		agent.standBy(find);
		
		targetPoint=null;
		previousDirection = null;
		
//		isTargetPointChanged = false;
//		isOtherRobotPointChanged = false;
	}
	
	/**
	 * Execution task
	 */
	public void perform() {
		//get current position
		try {
			if(gpsPort.isArrive()){
				currentPoint = (Point)gpsPort.get();
//				System.out.println("Acquired current point: "+currentPoint);
			}
		} catch (PortException e) {
			e.printStackTrace();
		}

		//get target object's type from the other control loops
		if(targetType==null){
			try {
				if(providedPort.isArrive()){
					targetType = (String)providedPort.get();
					currentComponent = find;
					targetPoint = null;
				}
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		else{
			if(currentComponent.getName().equals("Find")){
				if(targetPoint==null){//determine the target point
					if(find.mode == Mode.WAITING){
						System.out.println("Component: ApproachObject");
						connectPorts(findPort, find.getProvidedPorts().get(0));
						try {
							findPort.put((Object)targetType);
						} catch (PortException e) {
							e.printStackTrace();
						}
						find.activate();
					}
					if(find.mode==Mode.ACHIEVED){
						System.out.println("Find is finished.");
						targetPoint = null;
						if(findPort.isArrive()){
							try {
								targetPoint = (Point)findPort.get();
								if(targetPoint!=null){
									System.out.println("\t"+targetType+" is found at ("+targetPoint.x+","+targetPoint.y+").");
									previousDirection = null;
									find.passivate();
								}
								else{
									System.out.println("\t"+targetType+" is not found.");
								}
							} catch (PortException e) {
								e.printStackTrace();
							}
						}
					}
				}
				else{//already determined the target point
					//look for the obstacles
					if(find.mode == Mode.WAITING){
						System.out.println("Search obstacle");
						connectPorts(findPort, find.getProvidedPorts().get(0));
						try {
							findPort.put((Object)"Obstacle");
						} catch (PortException e) {
							e.printStackTrace();
						}
						find.activate();
					}
					if(find.mode==Mode.ACHIEVED || find.mode==Mode.NOT_ACHIEVED){
							
						obstaclePoint = null;
						if(findPort.isArrive()){
							try {
								Point tmpObstaclePoint = (Point)findPort.get();
								if(tmpObstaclePoint!=null && 
										(obstaclePoint==null || !obstaclePoint.equals(tmpObstaclePoint))){
								}
								obstaclePoint = tmpObstaclePoint;
								avoidPoints = new ArrayList<Point>();
								avoidPoints.add(obstaclePoint);
								if(obstaclePoint!=null){
									System.out.println("\tObstacle is found at "+obstaclePoint.toString()+".");
									find.passivate();
								}
							} catch (PortException e) {
								e.printStackTrace();
							}
						}
						
						//determine the next direction to move
						if(avoidPoints.contains(targetPoint)){
							//judge that the approaching object is failed because the target point is one of the places located the obstacles
							this.mode = Mode.NOT_ACHIEVED;
						}
						else{
//							Point currentPoint = simulator.getCurrentPoint();
							if(!targetPoint.equals(currentPoint)){

								//Selects possible directions by excluding the direction to the neighbor obstacles
								possibleDirection = getPossibleDirections(currentPoint);

								//Moves to the neighbor point if the point is the target place
								chooseNeighborMove(currentPoint, targetPoint);

								if(possibleDirection.size()>=2){
									// exclude the opposite direction to the destination
									if(currentPoint.x < targetPoint.x) possibleDirection.remove(Direction.WEST);
									if(currentPoint.x > targetPoint.x) possibleDirection.remove(Direction.EAST);
									if(currentPoint.y < targetPoint.y) possibleDirection.remove(Direction.SOUTH);
									if(currentPoint.y > targetPoint.y) possibleDirection.remove(Direction.NORTH);

									//prevent to go backward
									removeBackward();
								}
								
								connectPorts(moveToPort, moveTo.getProvidedPorts().get(0));
								try {
//									moveToPort.put(new TargetAndAvoidPoints(targetPoint, avoidPoints));

								switch(possibleDirection.size()){
								case 4:
									System.out.println("There are too many possible directions. "+possibleDirection.size());
									previousDirection = possibleDirection.get(0);
									break;
								case 3:
									System.out.println("There are three possible directions. ");
									previousDirection = findBestDirectionAmongThreeCandidates(currentPoint, targetPoint);
									moveToPort.put(determineMilestone(currentPoint, previousDirection));
									break;
								case 2:
									if((Math.random())*2>1){
										previousDirection = possibleDirection.get(0);
										moveToPort.put(determineMilestone(currentPoint, previousDirection));
									}
									else{
										previousDirection = possibleDirection.get(1);
										moveToPort.put(determineMilestone(currentPoint, previousDirection));
									}
									break;
								case 1:
									previousDirection = possibleDirection.get(0);
									moveToPort.put(determineMilestone(currentPoint, previousDirection));
									break;
								case 0:
									this.mode = Mode.NOT_ACHIEVED;
									System.out.println("There are no choise to go forward.");
									break;
								}
								} catch (PortException e) {
									e.printStackTrace();
								}

							}
							if(targetPoint.equals(currentPoint)){
								mode = Mode.ACHIEVED;
							}
						}
						
						currentComponent = moveTo;
						moveTo.activate();
					}
				}
			}
			if(currentComponent.getName().equals("MoveTo")){
				if(currentComponent.mode == Mode.ACHIEVED){
					if(currentPoint.x==targetPoint.x && currentPoint.y==targetPoint.y){
						mode = Mode.ACHIEVED;
						System.out.println("Arrive at ("+currentPoint.x + ","+currentPoint.y+").");
					}
					else {
						//an obstacle is on the target point, or the robot continues to move
						if(avoidPoints.contains(targetPoint)){
							//an obstacle is on the target point
							mode= Mode.NOT_ACHIEVED;
							moveTo.passivate();
							find.passivate();
						}
						else{//continue to move
							moveTo.passivate();
							find.passivate();
							currentComponent = find;
						}
						
					}
				}
			}
		}
	}
	
	/** Passivates the component's service */
	public void passivate(){
		super.passivate();
		targetType = null;
		targetPoint = null;
		find.passivate();
		moveTo.passivate();
		
	}
	
	
	/////////////////////////////  private methods for determining direction  /////////////////////////////
	
	/**
	 * Selects possible directions by excluding the direction to the neighbor obstacles
	 * @param p: source place (e.g. current position)
	 * @return list of possible directions to move
	 */
	private List<Direction> getPossibleDirections(Point p){
		List<Direction> resultList = new ArrayList<Direction>();
		if(!avoidPoints.contains(new Point(p.x+1, p.y))) resultList.add(Direction.EAST);
		if(!avoidPoints.contains(new Point(p.x-1, p.y))) resultList.add(Direction.WEST);
		if(!avoidPoints.contains(new Point(p.x, p.y+1))) resultList.add(Direction.NORTH);
		if(!avoidPoints.contains(new Point(p.x, p.y-1))) resultList.add(Direction.SOUTH);
		return resultList;
	}
	
	/**
	 * Determines the best direction among three candidate directions.
	 * @param from
	 * @param to
	 * @return
	 */
	private Direction findBestDirectionAmongThreeCandidates(Point from, Point to){
		if(from.x == to.x){
			if(from.y < to.y) return Direction.NORTH;
			if(from.y > to.y) return Direction.SOUTH;
		}
		else{
			if(from.y == to.y){
				if(from.x < to.x) return Direction.EAST;
				else return Direction.WEST;
			}
		}
		return Direction.STOP;
	}
	
	/**
	 *	Prevents to go backward by deleting the corresponding direction.
	 */
	private void removeBackward(){
		if(previousDirection!=null && possibleDirection.size()>=2){
			switch(previousDirection){
			case EAST:
				possibleDirection.remove(Direction.WEST);
				break;
			case WEST:
				possibleDirection.remove(Direction.EAST);
				break;
			case NORTH:
				possibleDirection.remove(Direction.SOUTH);
				break;
			case SOUTH:
				possibleDirection.remove(Direction.NORTH);
				break;
				default:
					System.out.println();
			}
		}
	}
	
	/**
	 * Moves to the neighbor point if the point is the target place.
	 */
	private void chooseNeighborMove(Point from, Point to){
		if(from.x == to.x){
			if(from.y + 1 == to.y && possibleDirection.contains(Direction.NORTH)){
				possibleDirection.clear();
				possibleDirection.add(Direction.NORTH);
			}
			else{
				if(from.y - 1 == to.y && possibleDirection.contains(Direction.SOUTH)){
					possibleDirection.clear();
					possibleDirection.add(Direction.SOUTH);
				}
			}
		}
		else{
			if(from.y == to.y){
				if(from.x + 1 == to.x && possibleDirection.contains(Direction.EAST)){
					possibleDirection.clear();
					possibleDirection.add(Direction.EAST);
				}
				else{
					if(from.x - 1 == to.x && possibleDirection.contains(Direction.WEST)){
						possibleDirection.clear();
						possibleDirection.add(Direction.WEST);
					}
				}
			}
		}
	}
	
	/**
	 * Determines the neighbor point to move.
	 */
	private Point determineMilestone(Point current, Direction d){
		switch (d) {
		case EAST:
			return(new Point(current.x+1, current.y));
		case WEST:
			return(new Point(current.x-1, current.y));
		case NORTH:
			return(new Point(current.x, current.y+1));
		case SOUTH:
			return(new Point(current.x, current.y-1));
		default:
			return null;
		}
	}
}
