package robot.de.comp.alternative;

import java.awt.Point;

import org.dyncle.ComponentBehaviour;
import org.dyncle.Mode;
import org.dyncle.Port;
import org.dyncle.PortException;
import org.dyncle.PortType;
import org.dyncle.SASystem;
import org.nak.cleaning.Direction;
import org.nak.cleaning.Simulator;

import robot.de.CleaningRobot;

/**
 *  Component for moving to the target<br/>
 * @author Hiroyuki NAKAGAWA
 */
@SuppressWarnings("serial")
public class MoveTo extends ComponentBehaviour {
	Point target; //destination
	Port<Point> targetPort; //Port
	
	Simulator simulator;
	// when this component is activate,  it prints out the component name just once
	boolean forPrint=false;

	public boolean done(){
		return false;
	}

	/**
	 * Constructor
	 */
	public MoveTo(SASystem agent) {
		super(agent, "MoveTo", "MoveTo");
		name = "MoveTo";
		targetPort = new Port<Point>("TargetPoint", PortType.PROVIDED, this);
		this.addPort(targetPort);
		simulator =((CleaningRobot)myAgent).simulator;
		forPrint = true;
	}

	/**
	 * Execution task
	 */
	public void perform() {
		if(forPrint){
			System.out.println("Component: Move");
			forPrint = false;
		}
		if(!targetPort.isEmpty()){//change the destination
			try {
				target = targetPort.get();
			} catch (PortException e) {
				e.printStackTrace();
			}
		}
		if(target==null){
			//no need to move more
			this.mode = Mode.ACHIEVED;
		}
		else{
			if(!target.equals(simulator.getCurrentPoint())){
				if((Math.random())*2>1){
					if(target.x != simulator.getCurrentPoint().x){
						if(simulator.getCurrentPoint().x < target.x){
							simulator.moveNext(Direction.EAST);
						}
						else{
							simulator.moveNext(Direction.WEST);
						}
					}
				}
				else{
					if(target.y != simulator.getCurrentPoint().y){
						if(simulator.getCurrentPoint().y < target.y){
							simulator.moveNext(Direction.NORTH);
						}
						else{
							simulator.moveNext(Direction.SOUTH);
						}
					}
				}
			}
			if(target.equals(simulator.getCurrentPoint())){
				mode = Mode.ACHIEVED;
			}
		}
	}
	
	/** Activates the component's service */
	public void activate(){
		super.activate();
		forPrint = true;
	}
	
}
